library angular2.change_detection;

export './src/change_detection/parser/ast.dart' show AST;
export './src/change_detection/parser/lexer.dart' show Lexer;
export './src/change_detection/parser/parser.dart' show Parser;
export './src/change_detection/parser/locals.dart' show Locals;
export './src/change_detection/exceptions.dart'
    show ExpressionChangedAfterItHasBeenChecked, ChangeDetectionError;
export './src/change_detection/interfaces.dart'
    show
        ChangeRecord,
        ChangeDispatcher,
        ChangeDetector,
        CHECK_ONCE,
        CHECK_ALWAYS,
        DETACHED,
        CHECKED;
export './src/change_detection/proto_change_detector.dart'
    show
        ProtoChangeDetector,
        DynamicProtoChangeDetector,
        JitProtoChangeDetector,
        BindingRecord;
export './src/change_detection/dynamic_change_detector.dart'
    show DynamicChangeDetector;
export './src/change_detection/pipes/pipe_registry.dart';
export './src/change_detection/change_detection_util.dart' show uninitialized;
export './src/change_detection/pipes/pipe.dart';
import './src/change_detection/proto_change_detector.dart'
    show
        ProtoChangeDetector,
        DynamicProtoChangeDetector,
        JitProtoChangeDetector;
import './src/change_detection/pipes/pipe_registry.dart' show PipeRegistry;
import './src/change_detection/pipes/iterable_changes.dart'
    show IterableChangesFactory;
import './src/change_detection/pipes/keyvalue_changes.dart'
    show KeyValueChangesFactory;
import './src/change_detection/pipes/null_pipe.dart' show NullPipeFactory;

class ChangeDetection {
  ProtoChangeDetector createProtoChangeDetector(String name) {
    return null;
  }
}
var defaultPipes = {
  "iterableDiff": [new IterableChangesFactory(), new NullPipeFactory()],
  "keyValDiff": [new KeyValueChangesFactory(), new NullPipeFactory()]
};
class DynamicChangeDetection extends ChangeDetection {
  PipeRegistry registry;
  DynamicChangeDetection(PipeRegistry registry) : super() {
    this.registry = registry;
  }
  ProtoChangeDetector createProtoChangeDetector(String name) {
    return new DynamicProtoChangeDetector(this.registry);
  }
}
class JitChangeDetection extends ChangeDetection {
  PipeRegistry registry;
  JitChangeDetection(PipeRegistry registry) : super() {
    this.registry = registry;
  }
  ProtoChangeDetector createProtoChangeDetector(String name) {
    return new JitProtoChangeDetector(this.registry);
  }
}
var _registry = new PipeRegistry(defaultPipes);
var dynamicChangeDetection = new DynamicChangeDetection(_registry);
var jitChangeDetection = new JitChangeDetection(_registry);
