library angular2.src.change_detection.abstract_change_detector;

import 'package:angular2/src/facade/lang.dart' show isPresent;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper;
import './interfaces.dart'
    show ChangeDetector, CHECK_ALWAYS, CHECK_ONCE, CHECKED, DETACHED;

class AbstractChangeDetector extends ChangeDetector {
  List children;
  ChangeDetector parent;
  String mode;
  AbstractChangeDetector() : super() {
    this.children = [];
    this.mode = CHECK_ALWAYS;
  }
  addChild(ChangeDetector cd) {
    ListWrapper.push(this.children, cd);
    cd.parent = this;
  }
  removeChild(ChangeDetector cd) {
    ListWrapper.remove(this.children, cd);
  }
  remove() {
    this.parent.removeChild(this);
  }
  detectChanges() {
    this._detectChanges(false);
  }
  checkNoChanges() {
    this._detectChanges(true);
  }
  _detectChanges(bool throwOnChange) {
    if (identical(this.mode, DETACHED) || identical(this.mode, CHECKED)) return;
    this.detectChangesInRecords(throwOnChange);
    this._detectChangesInChildren(throwOnChange);
    if (identical(this.mode, CHECK_ONCE)) this.mode = CHECKED;
  }
  detectChangesInRecords(bool throwOnChange) {}
  _detectChangesInChildren(bool throwOnChange) {
    var children = this.children;
    for (var i = 0; i < children.length; ++i) {
      children[i]._detectChanges(throwOnChange);
    }
  }
  markPathToRootAsCheckOnce() {
    var c = this;
    while (isPresent(c) && c.mode != DETACHED) {
      if (identical(c.mode, CHECKED)) c.mode = CHECK_ONCE;
      c = c.parent;
    }
  }
}
