library angular2.src.change_detection.coalesce;

import 'package:angular2/src/facade/lang.dart' show isPresent;
import 'package:angular2/src/facade/collection.dart'
    show List, ListWrapper, Map, MapWrapper;
import './proto_record.dart' show RECORD_TYPE_SELF, ProtoRecord;

List<ProtoRecord> coalesce(List<ProtoRecord> records) {
  var res = ListWrapper.create();
  var indexMap = MapWrapper.create();
  for (var i = 0; i < records.length; ++i) {
    var r = records[i];
    var record = _replaceIndices(r, res.length + 1, indexMap);
    var matchingRecord = _findMatching(record, res);
    if (isPresent(matchingRecord) && record.lastInBinding) {
      ListWrapper.push(
          res, _selfRecord(record, matchingRecord.selfIndex, res.length + 1));
      MapWrapper.set(indexMap, r.selfIndex, matchingRecord.selfIndex);
    } else if (isPresent(matchingRecord) && !record.lastInBinding) {
      MapWrapper.set(indexMap, r.selfIndex, matchingRecord.selfIndex);
    } else {
      ListWrapper.push(res, record);
      MapWrapper.set(indexMap, r.selfIndex, record.selfIndex);
    }
  }
  return res;
}
ProtoRecord _selfRecord(ProtoRecord r, num contextIndex, num selfIndex) {
  return new ProtoRecord(RECORD_TYPE_SELF, "self", null, [], r.fixedArgs,
      contextIndex, selfIndex, r.bindingMemento, r.directiveMemento,
      r.expressionAsString, r.lastInBinding, r.lastInDirective);
}
_findMatching(ProtoRecord r, List<ProtoRecord> rs) {
  return ListWrapper.find(rs, (rr) {
    return identical(rr.mode, r.mode) &&
        identical(rr.funcOrValue, r.funcOrValue) &&
        identical(rr.contextIndex, r.contextIndex) &&
        ListWrapper.equals(rr.args, r.args);
  });
}
_replaceIndices(ProtoRecord r, num selfIndex, Map indexMap) {
  var args = ListWrapper.map(r.args, (a) {
    return _map(indexMap, a);
  });
  var contextIndex = _map(indexMap, r.contextIndex);
  return new ProtoRecord(r.mode, r.name, r.funcOrValue, args, r.fixedArgs,
      contextIndex, selfIndex, r.bindingMemento, r.directiveMemento,
      r.expressionAsString, r.lastInBinding, r.lastInDirective);
}
_map(Map indexMap, num value) {
  var r = MapWrapper.get(indexMap, value);
  return isPresent(r) ? r : value;
}
