library angular2.src.change_detection.pipes.iterable_changes;

import 'package:angular2/src/facade/collection.dart'
    show isListLikeIterable, iterateListLike, ListWrapper, MapWrapper;
import 'package:angular2/src/facade/lang.dart'
    show int, isBlank, isPresent, stringify, getMapKey, looseIdentical;
import './pipe.dart' show NO_CHANGE, Pipe;

class IterableChangesFactory {
  bool supports(obj) {
    return IterableChanges.supportsObj(obj);
  }
  Pipe create() {
    return new IterableChanges();
  }
}
class IterableChanges extends Pipe {
  var _collection;
  int _length;
  _DuplicateMap _linkedRecords;
  _DuplicateMap _unlinkedRecords;
  CollectionChangeRecord _previousItHead;
  CollectionChangeRecord _itHead;
  CollectionChangeRecord _itTail;
  CollectionChangeRecord _additionsHead;
  CollectionChangeRecord _additionsTail;
  CollectionChangeRecord _movesHead;
  CollectionChangeRecord _movesTail;
  CollectionChangeRecord _removalsHead;
  CollectionChangeRecord _removalsTail;
  IterableChanges() : super() {
    this._collection = null;
    this._length = null;
    this._linkedRecords = null;
    this._unlinkedRecords = null;
    this._previousItHead = null;
    this._itHead = null;
    this._itTail = null;
    this._additionsHead = null;
    this._additionsTail = null;
    this._movesHead = null;
    this._movesTail = null;
    this._removalsHead = null;
    this._removalsTail = null;
  }
  static bool supportsObj(obj) {
    return isListLikeIterable(obj);
  }
  bool supports(obj) {
    return IterableChanges.supportsObj(obj);
  }
  get collection {
    return this._collection;
  }
  int get length {
    return this._length;
  }
  forEachItem(Function fn) {
    CollectionChangeRecord record;
    for (record = this._itHead;
        !identical(record, null);
        record = record._next) {
      fn(record);
    }
  }
  forEachPreviousItem(Function fn) {
    CollectionChangeRecord record;
    for (record = this._previousItHead;
        !identical(record, null);
        record = record._nextPrevious) {
      fn(record);
    }
  }
  forEachAddedItem(Function fn) {
    CollectionChangeRecord record;
    for (record = this._additionsHead;
        !identical(record, null);
        record = record._nextAdded) {
      fn(record);
    }
  }
  forEachMovedItem(Function fn) {
    CollectionChangeRecord record;
    for (record = this._movesHead;
        !identical(record, null);
        record = record._nextMoved) {
      fn(record);
    }
  }
  forEachRemovedItem(Function fn) {
    CollectionChangeRecord record;
    for (record = this._removalsHead;
        !identical(record, null);
        record = record._nextRemoved) {
      fn(record);
    }
  }
  transform(collection) {
    if (this.check(collection)) {
      return this;
    } else {
      return NO_CHANGE;
    }
  }
  bool check(collection) {
    this._reset();
    CollectionChangeRecord record = this._itHead;
    bool mayBeDirty = false;
    int index;
    var item;
    if (ListWrapper.isList(collection)) {
      var list = collection;
      this._length = collection.length;
      for (index = 0; index < this._length; index++) {
        item = list[index];
        if (identical(record, null) || !looseIdentical(record.item, item)) {
          record = this._mismatch(record, item, index);
          mayBeDirty = true;
        } else if (mayBeDirty) {
          record = this._verifyReinsertion(record, item, index);
        }
        record = record._next;
      }
    } else {
      index = 0;
      iterateListLike(collection, (item) {
        if (identical(record, null) || !looseIdentical(record.item, item)) {
          record = this._mismatch(record, item, index);
          mayBeDirty = true;
        } else if (mayBeDirty) {
          record = this._verifyReinsertion(record, item, index);
        }
        record = record._next;
        index++;
      });
      this._length = index;
    }
    this._truncate(record);
    this._collection = collection;
    return this.isDirty;
  }
  bool get isDirty {
    return !identical(this._additionsHead, null) ||
        !identical(this._movesHead, null) ||
        !identical(this._removalsHead, null);
  }
  _reset() {
    if (this.isDirty) {
      CollectionChangeRecord record;
      CollectionChangeRecord nextRecord;
      for (record = this._previousItHead = this._itHead;
          !identical(record, null);
          record = record._next) {
        record._nextPrevious = record._next;
      }
      for (record = this._additionsHead;
          !identical(record, null);
          record = record._nextAdded) {
        record.previousIndex = record.currentIndex;
      }
      this._additionsHead = this._additionsTail = null;
      for (record = this._movesHead;
          !identical(record, null);
          record = nextRecord) {
        record.previousIndex = record.currentIndex;
        nextRecord = record._nextMoved;
      }
      this._movesHead = this._movesTail = null;
      this._removalsHead = this._removalsTail = null;
    }
  }
  CollectionChangeRecord _mismatch(
      CollectionChangeRecord record, item, int index) {
    CollectionChangeRecord previousRecord;
    if (identical(record, null)) {
      previousRecord = this._itTail;
    } else {
      previousRecord = record._prev;
      this._remove(record);
    }
    record = identical(this._linkedRecords, null)
        ? null
        : this._linkedRecords.get(item, index);
    if (!identical(record, null)) {
      this._moveAfter(record, previousRecord, index);
    } else {
      record = identical(this._unlinkedRecords, null)
          ? null
          : this._unlinkedRecords.get(item);
      if (!identical(record, null)) {
        this._reinsertAfter(record, previousRecord, index);
      } else {
        record = this._addAfter(
            new CollectionChangeRecord(item), previousRecord, index);
      }
    }
    return record;
  }
  CollectionChangeRecord _verifyReinsertion(
      CollectionChangeRecord record, item, int index) {
    CollectionChangeRecord reinsertRecord = identical(
        this._unlinkedRecords, null) ? null : this._unlinkedRecords.get(item);
    if (!identical(reinsertRecord, null)) {
      record = this._reinsertAfter(reinsertRecord, record._prev, index);
    } else if (record.currentIndex != index) {
      record.currentIndex = index;
      this._addToMoves(record, index);
    }
    return record;
  }
  _truncate(CollectionChangeRecord record) {
    while (!identical(record, null)) {
      CollectionChangeRecord nextRecord = record._next;
      this._addToRemovals(this._unlink(record));
      record = nextRecord;
    }
    if (!identical(this._unlinkedRecords, null)) {
      this._unlinkedRecords.clear();
    }
    if (!identical(this._additionsTail, null)) {
      this._additionsTail._nextAdded = null;
    }
    if (!identical(this._movesTail, null)) {
      this._movesTail._nextMoved = null;
    }
    if (!identical(this._itTail, null)) {
      this._itTail._next = null;
    }
    if (!identical(this._removalsTail, null)) {
      this._removalsTail._nextRemoved = null;
    }
  }
  CollectionChangeRecord _reinsertAfter(CollectionChangeRecord record,
      CollectionChangeRecord prevRecord, int index) {
    if (!identical(this._unlinkedRecords, null)) {
      this._unlinkedRecords.remove(record);
    }
    var prev = record._prevRemoved;
    var next = record._nextRemoved;
    if (identical(prev, null)) {
      this._removalsHead = next;
    } else {
      prev._nextRemoved = next;
    }
    if (identical(next, null)) {
      this._removalsTail = prev;
    } else {
      next._prevRemoved = prev;
    }
    this._insertAfter(record, prevRecord, index);
    this._addToMoves(record, index);
    return record;
  }
  CollectionChangeRecord _moveAfter(CollectionChangeRecord record,
      CollectionChangeRecord prevRecord, int index) {
    this._unlink(record);
    this._insertAfter(record, prevRecord, index);
    this._addToMoves(record, index);
    return record;
  }
  CollectionChangeRecord _addAfter(CollectionChangeRecord record,
      CollectionChangeRecord prevRecord, int index) {
    this._insertAfter(record, prevRecord, index);
    if (identical(this._additionsTail, null)) {
      this._additionsTail = this._additionsHead = record;
    } else {
      this._additionsTail = this._additionsTail._nextAdded = record;
    }
    return record;
  }
  CollectionChangeRecord _insertAfter(CollectionChangeRecord record,
      CollectionChangeRecord prevRecord, int index) {
    CollectionChangeRecord next =
        identical(prevRecord, null) ? this._itHead : prevRecord._next;
    record._next = next;
    record._prev = prevRecord;
    if (identical(next, null)) {
      this._itTail = record;
    } else {
      next._prev = record;
    }
    if (identical(prevRecord, null)) {
      this._itHead = record;
    } else {
      prevRecord._next = record;
    }
    if (identical(this._linkedRecords, null)) {
      this._linkedRecords = new _DuplicateMap();
    }
    this._linkedRecords.put(record);
    record.currentIndex = index;
    return record;
  }
  CollectionChangeRecord _remove(CollectionChangeRecord record) {
    return this._addToRemovals(this._unlink(record));
  }
  CollectionChangeRecord _unlink(CollectionChangeRecord record) {
    if (!identical(this._linkedRecords, null)) {
      this._linkedRecords.remove(record);
    }
    var prev = record._prev;
    var next = record._next;
    if (identical(prev, null)) {
      this._itHead = next;
    } else {
      prev._next = next;
    }
    if (identical(next, null)) {
      this._itTail = prev;
    } else {
      next._prev = prev;
    }
    return record;
  }
  CollectionChangeRecord _addToMoves(
      CollectionChangeRecord record, int toIndex) {
    if (identical(record.previousIndex, toIndex)) {
      return record;
    }
    if (identical(this._movesTail, null)) {
      this._movesTail = this._movesHead = record;
    } else {
      this._movesTail = this._movesTail._nextMoved = record;
    }
    return record;
  }
  CollectionChangeRecord _addToRemovals(CollectionChangeRecord record) {
    if (identical(this._unlinkedRecords, null)) {
      this._unlinkedRecords = new _DuplicateMap();
    }
    this._unlinkedRecords.put(record);
    record.currentIndex = null;
    record._nextRemoved = null;
    if (identical(this._removalsTail, null)) {
      this._removalsTail = this._removalsHead = record;
      record._prevRemoved = null;
    } else {
      record._prevRemoved = this._removalsTail;
      this._removalsTail = this._removalsTail._nextRemoved = record;
    }
    return record;
  }
  String toString() {
    CollectionChangeRecord record;
    var list = [];
    for (record = this._itHead;
        !identical(record, null);
        record = record._next) {
      ListWrapper.push(list, record);
    }
    var previous = [];
    for (record = this._previousItHead;
        !identical(record, null);
        record = record._nextPrevious) {
      ListWrapper.push(previous, record);
    }
    var additions = [];
    for (record = this._additionsHead;
        !identical(record, null);
        record = record._nextAdded) {
      ListWrapper.push(additions, record);
    }
    var moves = [];
    for (record = this._movesHead;
        !identical(record, null);
        record = record._nextMoved) {
      ListWrapper.push(moves, record);
    }
    var removals = [];
    for (record = this._removalsHead;
        !identical(record, null);
        record = record._nextRemoved) {
      ListWrapper.push(removals, record);
    }
    return "collection: " +
        list.join(', ') +
        "\n" +
        "previous: " +
        previous.join(', ') +
        "\n" +
        "additions: " +
        additions.join(', ') +
        "\n" +
        "moves: " +
        moves.join(', ') +
        "\n" +
        "removals: " +
        removals.join(', ') +
        "\n";
  }
}
class CollectionChangeRecord {
  int currentIndex;
  int previousIndex;
  var item;
  CollectionChangeRecord _nextPrevious;
  CollectionChangeRecord _prev;
  CollectionChangeRecord _next;
  CollectionChangeRecord _prevDup;
  CollectionChangeRecord _nextDup;
  CollectionChangeRecord _prevRemoved;
  CollectionChangeRecord _nextRemoved;
  CollectionChangeRecord _nextAdded;
  CollectionChangeRecord _nextMoved;
  CollectionChangeRecord(item) {
    this.currentIndex = null;
    this.previousIndex = null;
    this.item = item;
    this._nextPrevious = null;
    this._prev = null;
    this._next = null;
    this._prevDup = null;
    this._nextDup = null;
    this._prevRemoved = null;
    this._nextRemoved = null;
    this._nextAdded = null;
    this._nextMoved = null;
  }
  String toString() {
    return identical(this.previousIndex, this.currentIndex)
        ? stringify(this.item)
        : stringify(this.item) +
            '[' +
            stringify(this.previousIndex) +
            '->' +
            stringify(this.currentIndex) +
            ']';
  }
}
class _DuplicateItemRecordList {
  CollectionChangeRecord _head;
  CollectionChangeRecord _tail;
  _DuplicateItemRecordList() {
    this._head = null;
    this._tail = null;
  }
  add(CollectionChangeRecord record) {
    if (identical(this._head, null)) {
      this._head = this._tail = record;
      record._nextDup = null;
      record._prevDup = null;
    } else {
      this._tail._nextDup = record;
      record._prevDup = this._tail;
      record._nextDup = null;
      this._tail = record;
    }
  }
  CollectionChangeRecord get(item, int afterIndex) {
    CollectionChangeRecord record;
    for (record = this._head;
        !identical(record, null);
        record = record._nextDup) {
      if ((identical(afterIndex, null) || afterIndex < record.currentIndex) &&
          looseIdentical(record.item, item)) {
        return record;
      }
    }
    return null;
  }
  bool remove(CollectionChangeRecord record) {
    CollectionChangeRecord prev = record._prevDup;
    CollectionChangeRecord next = record._nextDup;
    if (identical(prev, null)) {
      this._head = next;
    } else {
      prev._nextDup = next;
    }
    if (identical(next, null)) {
      this._tail = prev;
    } else {
      next._prevDup = prev;
    }
    return identical(this._head, null);
  }
}
class _DuplicateMap {
  Map map;
  _DuplicateMap() {
    this.map = MapWrapper.create();
  }
  put(CollectionChangeRecord record) {
    var key = getMapKey(record.item);
    var duplicates = MapWrapper.get(this.map, key);
    if (!isPresent(duplicates)) {
      duplicates = new _DuplicateItemRecordList();
      MapWrapper.set(this.map, key, duplicates);
    }
    duplicates.add(record);
  }
  CollectionChangeRecord get(value, [afterIndex = null]) {
    var key = getMapKey(value);
    var recordList = MapWrapper.get(this.map, key);
    return isBlank(recordList) ? null : recordList.get(value, afterIndex);
  }
  CollectionChangeRecord remove(CollectionChangeRecord record) {
    var key = getMapKey(record.item);
    _DuplicateItemRecordList recordList = MapWrapper.get(this.map, key);
    if (recordList.remove(record)) {
      MapWrapper.delete(this.map, key);
    }
    return record;
  }
  bool get isEmpty {
    return identical(MapWrapper.size(this.map), 0);
  }
  clear() {
    MapWrapper.clear(this.map);
  }
  String toString() {
    return '_DuplicateMap(' + stringify(this.map) + ')';
  }
}
