library angular2.src.core.application;

import 'package:angular2/di.dart' show Injector, bind, OpaqueToken;
import 'package:angular2/src/facade/lang.dart'
    show
        Type,
        isBlank,
        isPresent,
        BaseException,
        assertionsEnabled,
        print,
        stringify;
import 'package:angular2/src/dom/browser_adapter.dart' show BrowserDomAdapter;
import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import './compiler/compiler.dart' show Compiler, CompilerCache;
import './compiler/view.dart' show ProtoView;
import 'package:angular2/src/reflection/reflection.dart'
    show Reflector, reflector;
import 'package:angular2/change_detection.dart'
    show
        Parser,
        Lexer,
        ChangeDetection,
        dynamicChangeDetection,
        jitChangeDetection;
import './exception_handler.dart' show ExceptionHandler;
import './compiler/template_loader.dart' show TemplateLoader;
import './compiler/template_resolver.dart' show TemplateResolver;
import './compiler/directive_metadata_reader.dart' show DirectiveMetadataReader;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper;
import 'package:angular2/src/facade/async.dart' show Future, PromiseWrapper;
import 'package:angular2/src/core/zone/vm_turn_zone.dart' show VmTurnZone;
import 'package:angular2/src/core/life_cycle/life_cycle.dart' show LifeCycle;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show
        ShadowDomStrategy,
        NativeShadowDomStrategy,
        EmulatedUnscopedShadowDomStrategy;
import 'package:angular2/src/core/compiler/xhr/xhr.dart' show XHR;
import 'package:angular2/src/core/compiler/xhr/xhr_impl.dart' show XHRImpl;
import 'package:angular2/src/core/events/event_manager.dart'
    show EventManager, DomEventsPlugin;
import 'package:angular2/src/core/events/hammer_gestures.dart'
    show HammerGesturesPlugin;
import 'package:angular2/src/di/binding.dart' show Binding;
import 'package:angular2/src/core/compiler/component_url_mapper.dart'
    show ComponentUrlMapper;
import 'package:angular2/src/core/compiler/url_resolver.dart' show UrlResolver;
import 'package:angular2/src/core/compiler/style_url_resolver.dart'
    show StyleUrlResolver;
import 'package:angular2/src/core/compiler/style_inliner.dart'
    show StyleInliner;
import 'package:angular2/src/core/compiler/css_processor.dart'
    show CssProcessor;
import 'package:angular2/src/core/annotations/annotations.dart' show Component;

Injector _rootInjector;
var _rootBindings = [bind(Reflector).toValue(reflector)];
var appViewToken = new OpaqueToken('AppView');
var appChangeDetectorToken = new OpaqueToken('AppChangeDetector');
var appElementToken = new OpaqueToken('AppElement');
var appComponentAnnotatedTypeToken =
    new OpaqueToken('AppComponentAnnotatedType');
var appDocumentToken = new OpaqueToken('AppDocument');
List<Binding> _injectorBindings(appComponentType) {
  return [
    bind(appDocumentToken).toValue(DOM.defaultDoc()),
    bind(appComponentAnnotatedTypeToken).toFactory((reader) {
      return reader.read(appComponentType);
    }, [DirectiveMetadataReader]),
    bind(appElementToken).toFactory((appComponentAnnotatedType, appDocument) {
      var selector = appComponentAnnotatedType.annotation.selector;
      var element = DOM.querySelector(appDocument, selector);
      if (isBlank(element)) {
        throw new BaseException(
            '''The app selector \"${selector}\" did not match any elements''');
      }
      return element;
    }, [appComponentAnnotatedTypeToken, appDocumentToken]),
    bind(appViewToken).toAsyncFactory((changeDetection, compiler, injector,
        appElement, appComponentAnnotatedType, strategy, eventManager) {
      var annotation = appComponentAnnotatedType.annotation;
      if (!isBlank(annotation) && !(annotation is Component)) {
        var type = appComponentAnnotatedType.type;
        throw new BaseException('''Only Components can be bootstrapped; ''' +
            '''Directive of ${stringify(type)} is not a Component''');
      }
      return compiler.compile(appComponentAnnotatedType.type).then((protoView) {
        var appProtoView = ProtoView.createRootProtoView(protoView, appElement,
            appComponentAnnotatedType,
            changeDetection.createProtoChangeDetector('root'), strategy);
        var view = appProtoView.instantiate(null, eventManager);
        view.hydrate(injector, null, null, new Object(), null);
        return view;
      });
    }, [
      ChangeDetection,
      Compiler,
      Injector,
      appElementToken,
      appComponentAnnotatedTypeToken,
      ShadowDomStrategy,
      EventManager
    ]),
    bind(appChangeDetectorToken).toFactory((rootView) {
      return rootView.changeDetector;
    }, [appViewToken]),
    bind(appComponentType).toFactory((rootView) {
      return rootView.elementInjectors[0].getComponent();
    }, [appViewToken]),
    bind(LifeCycle).toFactory((exceptionHandler) {
      return new LifeCycle(exceptionHandler, null, assertionsEnabled());
    }, [ExceptionHandler]),
    bind(EventManager).toFactory((zone) {
      var plugins = [new HammerGesturesPlugin(), new DomEventsPlugin()];
      return new EventManager(plugins, zone);
    }, [VmTurnZone]),
    bind(ShadowDomStrategy).toFactory((styleUrlResolver, doc) {
      return new EmulatedUnscopedShadowDomStrategy(styleUrlResolver, doc.head);
    }, [StyleUrlResolver, appDocumentToken]),
    Compiler,
    CompilerCache,
    TemplateResolver,
    bind(ChangeDetection).toValue(dynamicChangeDetection),
    TemplateLoader,
    DirectiveMetadataReader,
    Parser,
    Lexer,
    ExceptionHandler,
    bind(XHR).toValue(new XHRImpl()),
    ComponentUrlMapper,
    UrlResolver,
    StyleUrlResolver,
    StyleInliner,
    bind(CssProcessor).toFactory(() {
      return new CssProcessor(null);
    }, [])
  ];
}
VmTurnZone _createVmZone(Function givenReporter) {
  var defaultErrorReporter = (exception, stackTrace) {
    var longStackTrace =
        ListWrapper.join(stackTrace, "\n\n-----async gap-----\n");
    print('''${exception}\n\n${longStackTrace}''');
    throw exception;
  };
  var reporter =
      isPresent(givenReporter) ? givenReporter : defaultErrorReporter;
  var zone = new VmTurnZone(enableLongStackTrace: assertionsEnabled());
  zone.initCallbacks(onErrorHandler: reporter);
  return zone;
}
Future<Injector> bootstrap(Type appComponentType,
    [List<Binding> componentServiceBindings = null,
    Function errorReporter = null]) {
  BrowserDomAdapter.makeCurrent();
  var bootstrapProcess = PromiseWrapper.completer();
  var zone = _createVmZone(errorReporter);
  zone.run(() {
    var appInjector =
        _createAppInjector(appComponentType, componentServiceBindings, zone);
    PromiseWrapper.then(appInjector.asyncGet(appViewToken), (rootView) {
      var lc = appInjector.get(LifeCycle);
      lc.registerWith(zone, rootView.changeDetector);
      lc.tick();
      bootstrapProcess.resolve(appInjector);
    }, (err) {
      bootstrapProcess.reject(err);
    });
  });
  return bootstrapProcess.promise;
}
Injector _createAppInjector(
    Type appComponentType, List<Binding> bindings, VmTurnZone zone) {
  if (isBlank(_rootInjector)) _rootInjector = new Injector(_rootBindings);
  var mergedBindings = isPresent(bindings)
      ? ListWrapper.concat(_injectorBindings(appComponentType), bindings)
      : _injectorBindings(appComponentType);
  ListWrapper.push(mergedBindings, bind(VmTurnZone).toValue(zone));
  return _rootInjector.createChild(mergedBindings);
}
