library angular2.src.core.compiler.css_processor;

import 'package:angular2/di.dart' show Injectable;
import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import 'package:angular2/src/facade/lang.dart' show isPresent;
import 'package:angular2/src/facade/collection.dart' show List;
import './pipeline/compile_step.dart' show CompileStep;
import './pipeline/compile_element.dart' show CompileElement;
import './pipeline/compile_control.dart' show CompileControl;
import './shadow_dom_strategy.dart' show ShadowDomStrategy;
import './directive_metadata.dart' show DirectiveMetadata;

@Injectable()
class CssProcessor {
  List<CssTransformer> _transformers;
  CssProcessor(List<CssTransformer> transformers) {
    this._transformers = transformers;
  }
  getCompileStep(DirectiveMetadata cmpMetadata,
      ShadowDomStrategy shadowDomStrategy, String templateUrl) {
    var strategyStep =
        shadowDomStrategy.getStyleCompileStep(cmpMetadata, templateUrl);
    return new _CssProcessorStep(strategyStep, this._transformers);
  }
}
class CssTransformer {
  transform(styleElement) {}
}
class _CssProcessorStep extends CompileStep {
  CompileStep _strategyStep;
  List<CssTransformer> _transformers;
  _CssProcessorStep(CompileStep strategyStep, List<CssTransformer> transformers)
      : super() {
    this._strategyStep = strategyStep;
    this._transformers = transformers;
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    if (DOM.tagName(current.element) == 'STYLE') {
      current.ignoreBindings = true;
      if (isPresent(this._transformers)) {
        var styleEl = current.element;
        for (var i = 0; i < this._transformers.length; i++) {
          this._transformers[i].transform(styleEl);
        }
      }
      if (isPresent(this._strategyStep)) {
        this._strategyStep.process(parent, current, control);
      }
    }
  }
}
