library angular2.src.core.compiler.pipeline.compile_pipeline;

import 'package:angular2/src/facade/lang.dart' show isPresent;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper;
import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import './compile_element.dart' show CompileElement;
import './compile_control.dart' show CompileControl;
import './compile_step.dart' show CompileStep;

class CompilePipeline {
  CompileControl _control;
  CompilePipeline(List<CompileStep> steps) {
    this._control = new CompileControl(steps);
  }
  List process(rootElement, [String compilationCtxtDescription = '']) {
    var results = ListWrapper.create();
    this._process(results, null, new CompileElement(
        rootElement, compilationCtxtDescription), compilationCtxtDescription);
    return results;
  }
  _process(results, CompileElement parent, CompileElement current,
      [String compilationCtxtDescription = '']) {
    var additionalChildren =
        this._control.internalProcess(results, 0, parent, current);
    if (current.compileChildren) {
      var node = DOM.firstChild(DOM.templateAwareRoot(current.element));
      while (isPresent(node)) {
        var nextNode = DOM.nextSibling(node);
        if (DOM.isElementNode(node)) {
          this._process(results, current,
              new CompileElement(node, compilationCtxtDescription));
        }
        node = nextNode;
      }
    }
    if (isPresent(additionalChildren)) {
      for (var i = 0; i < additionalChildren.length; i++) {
        this._process(results, current, additionalChildren[i]);
      }
    }
  }
}
