library angular2.src.core.compiler.pipeline.default_steps;

import 'package:angular2/change_detection.dart' show ChangeDetection, Parser;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper;
import './property_binding_parser.dart' show PropertyBindingParser;
import './text_interpolation_parser.dart' show TextInterpolationParser;
import './directive_parser.dart' show DirectiveParser;
import './view_splitter.dart' show ViewSplitter;
import './element_binding_marker.dart' show ElementBindingMarker;
import './proto_view_builder.dart' show ProtoViewBuilder;
import './proto_element_injector_builder.dart' show ProtoElementInjectorBuilder;
import './element_binder_builder.dart' show ElementBinderBuilder;
import 'package:angular2/src/core/compiler/css_processor.dart'
    show CssProcessor;
import 'package:angular2/src/core/compiler/directive_metadata.dart'
    show DirectiveMetadata;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show ShadowDomStrategy, EmulatedScopedShadowDomStrategy;

createDefaultSteps(ChangeDetection changeDetection, Parser parser,
    DirectiveMetadata compiledComponent, List<DirectiveMetadata> directives,
    ShadowDomStrategy shadowDomStrategy, String templateUrl,
    CssProcessor cssProcessor) {
  var steps = [
    new ViewSplitter(parser),
    cssProcessor.getCompileStep(
        compiledComponent, shadowDomStrategy, templateUrl),
    shadowDomStrategy.getTemplateCompileStep(compiledComponent),
    new PropertyBindingParser(parser),
    new DirectiveParser(directives),
    new TextInterpolationParser(parser),
    new ElementBindingMarker(),
    new ProtoViewBuilder(changeDetection, shadowDomStrategy),
    new ProtoElementInjectorBuilder(),
    new ElementBinderBuilder(parser)
  ];
  return steps;
}
