library angular2.src.core.compiler.pipeline.directive_parser;

import 'package:angular2/src/facade/lang.dart'
    show isPresent, isBlank, BaseException, assertionsEnabled, RegExpWrapper;
import 'package:angular2/src/facade/collection.dart' show List, MapWrapper;
import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import '../selector.dart' show SelectorMatcher;
import '../selector.dart' show CssSelector;
import '../directive_metadata.dart' show DirectiveMetadata;
import '../../annotations/annotations.dart'
    show DynamicComponent, Component, Viewport;
import './compile_step.dart' show CompileStep;
import './compile_element.dart' show CompileElement;
import './compile_control.dart' show CompileControl;

var PROPERTY_BINDING_REGEXP = RegExpWrapper.create('^ *([^\\s\\|]+)');
class DirectiveParser extends CompileStep {
  SelectorMatcher _selectorMatcher;
  DirectiveParser(List<DirectiveMetadata> directives) : super() {
    var selector;
    this._selectorMatcher = new SelectorMatcher();
    for (var i = 0; i < directives.length; i++) {
      var directiveMetadata = directives[i];
      selector = CssSelector.parse(directiveMetadata.annotation.selector);
      this._selectorMatcher.addSelectables(selector, directiveMetadata);
    }
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    var attrs = current.attrs();
    var classList = current.classList();
    var cssSelector = new CssSelector();
    var nodeName = DOM.nodeName(current.element);
    cssSelector.setElement(nodeName);
    for (var i = 0; i < classList.length; i++) {
      cssSelector.addClassName(classList[i]);
    }
    MapWrapper.forEach(attrs, (attrValue, attrName) {
      cssSelector.addAttribute(attrName, attrValue);
    });
    var isTemplateElement = DOM.isTemplateElement(current.element);
    this._selectorMatcher.match(cssSelector, (selector, directive) {
      current.addDirective(
          checkDirectiveValidity(directive, current, isTemplateElement));
    });
  }
}
checkDirectiveValidity(directive, current, isTemplateElement) {
  var isComponent = directive.annotation is Component ||
      directive.annotation is DynamicComponent;
  var alreadyHasComponent = isPresent(current.componentDirective);
  if (directive.annotation is Viewport) {
    if (!isTemplateElement) {
      throw new BaseException(
          '''Viewport directives need to be placed on <template> elements or elements ''' +
              '''with template attribute - check ${current.elementDescription}''');
    } else if (isPresent(current.viewportDirective)) {
      throw new BaseException(
          '''Only one viewport directive can be used per element - check ${current.elementDescription}''');
    }
  } else if (isTemplateElement) {
    throw new BaseException(
        '''Only template directives are allowed on template elements - check ${current.elementDescription}''');
  } else if (isComponent && alreadyHasComponent) {
    throw new BaseException(
        '''Multiple component directives not allowed on the same element - check ${current.elementDescription}''');
  }
  return directive;
}
