library angular2.src.core.compiler.pipeline.property_binding_parser;

import 'package:angular2/src/facade/lang.dart'
    show isPresent, isBlank, RegExpWrapper, BaseException;
import 'package:angular2/src/facade/collection.dart' show MapWrapper;
import 'package:angular2/change_detection.dart'
    show Parser, AST, ExpressionWithSource;
import './compile_step.dart' show CompileStep;
import './compile_element.dart' show CompileElement;
import './compile_control.dart' show CompileControl;

var BIND_NAME_REGEXP = RegExpWrapper.create(
    '^(?:(?:(?:(bind)|(var)|(on))-(.+))|\\[([^\\]]+)\\]|\\(([^\\)]+)\\)|(#)(.+))\$');
class PropertyBindingParser extends CompileStep {
  Parser _parser;
  PropertyBindingParser(Parser parser) : super() {
    this._parser = parser;
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    if (current.ignoreBindings) {
      return;
    }
    var attrs = current.attrs();
    var newAttrs = MapWrapper.create();
    var desc = current.elementDescription;
    MapWrapper.forEach(attrs, (attrValue, attrName) {
      var bindParts = RegExpWrapper.firstMatch(BIND_NAME_REGEXP, attrName);
      if (isPresent(bindParts)) {
        if (isPresent(bindParts[1])) {
          current.addPropertyBinding(
              bindParts[4], this._parseBinding(attrValue, desc));
          MapWrapper.set(newAttrs, bindParts[4], attrValue);
        } else if (isPresent(bindParts[2]) || isPresent(bindParts[7])) {
          var identifier = (isPresent(bindParts[4]) &&
              !identical(bindParts[4], '')) ? bindParts[4] : bindParts[8];
          var value = attrValue == '' ? '\$implicit' : attrValue;
          current.addVariableBinding(identifier, value);
          MapWrapper.set(newAttrs, identifier, value);
        } else if (isPresent(bindParts[3])) {
          current.addEventBinding(
              bindParts[4], this._parseAction(attrValue, desc));
        } else if (isPresent(bindParts[5])) {
          current.addPropertyBinding(
              bindParts[5], this._parseBinding(attrValue, desc));
          MapWrapper.set(newAttrs, bindParts[5], attrValue);
        } else if (isPresent(bindParts[6])) {
          current.addEventBinding(
              bindParts[6], this._parseAction(attrValue, desc));
        }
      } else {
        var ast = this._parseInterpolation(attrValue, desc);
        if (isPresent(ast)) {
          current.addPropertyBinding(attrName, ast);
        }
      }
    });
    MapWrapper.forEach(newAttrs, (attrValue, attrName) {
      MapWrapper.set(attrs, attrName, attrValue);
    });
  }
  AST _parseInterpolation(String input, String location) {
    return this._parser.parseInterpolation(input, location);
  }
  AST _parseBinding(String input, String location) {
    return this._parser.parseBinding(input, location);
  }
  AST _parseAction(String input, String location) {
    return this._parser.parseAction(input, location);
  }
}
