library angular2.src.core.compiler.pipeline.proto_element_injector_builder;

import 'package:angular2/src/facade/lang.dart' show isPresent, isBlank;
import 'package:angular2/src/facade/collection.dart'
    show ListWrapper, MapWrapper;
import '../element_injector.dart'
    show ProtoElementInjector, ComponentKeyMetaData, DirectiveBinding;
import './compile_step.dart' show CompileStep;
import './compile_element.dart' show CompileElement;
import './compile_control.dart' show CompileControl;
import '../directive_metadata.dart' show DirectiveMetadata;

class ProtoElementInjectorBuilder extends CompileStep {
  internalCreateProtoElementInjector(
      parent, index, directives, firstBindingIsComponent, distance) {
    return new ProtoElementInjector(
        parent, index, directives, firstBindingIsComponent, distance);
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    var distanceToParentInjector =
        this._getDistanceToParentInjector(parent, current);
    var parentProtoElementInjector =
        this._getParentProtoElementInjector(parent, current);
    var injectorBindings =
        ListWrapper.map(current.getAllDirectives(), this._createBinding);
    if (injectorBindings.length > 0 || isPresent(current.variableBindings)) {
      var protoView = current.inheritedProtoView;
      var hasComponent = isPresent(current.componentDirective);
      current.inheritedProtoElementInjector = this
          .internalCreateProtoElementInjector(parentProtoElementInjector,
              protoView.elementBinders.length, injectorBindings, hasComponent,
              distanceToParentInjector);
      current.distanceToParentInjector = 0;
      if (isPresent(current.variableBindings) &&
          !isPresent(current.viewportDirective)) {
        current.inheritedProtoElementInjector.exportComponent = hasComponent;
        current.inheritedProtoElementInjector.exportElement = !hasComponent;
        var exportImplicitName =
            MapWrapper.get(current.variableBindings, '\$implicit');
        if (isPresent(exportImplicitName)) {
          current.inheritedProtoElementInjector.exportImplicitName =
              exportImplicitName;
        }
      }
    } else {
      current.inheritedProtoElementInjector = parentProtoElementInjector;
      current.distanceToParentInjector = distanceToParentInjector;
    }
  }
  _getDistanceToParentInjector(parent, current) {
    return isPresent(parent) ? parent.distanceToParentInjector + 1 : 0;
  }
  _getParentProtoElementInjector(parent, current) {
    if (isPresent(parent) && !current.isViewRoot) {
      return parent.inheritedProtoElementInjector;
    }
    return null;
  }
  DirectiveBinding _createBinding(DirectiveMetadata d) {
    return DirectiveBinding.createFromType(d.type, d.annotation);
  }
}
