library angular2.src.core.compiler.pipeline.proto_view_builder;

import 'package:angular2/src/facade/lang.dart' show isPresent, BaseException;
import 'package:angular2/src/facade/collection.dart'
    show ListWrapper, MapWrapper;
import '../view.dart' show ProtoView;
import 'package:angular2/change_detection.dart' show ChangeDetection;
import './compile_step.dart' show CompileStep;
import './compile_element.dart' show CompileElement;
import './compile_control.dart' show CompileControl;
import '../shadow_dom_strategy.dart' show ShadowDomStrategy;

class ProtoViewBuilder extends CompileStep {
  ChangeDetection changeDetection;
  ShadowDomStrategy _shadowDomStrategy;
  ProtoViewBuilder(
      ChangeDetection changeDetection, ShadowDomStrategy shadowDomStrategy)
      : super() {
    this._shadowDomStrategy = shadowDomStrategy;
    this.changeDetection = changeDetection;
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    var inheritedProtoView = null;
    if (current.isViewRoot) {
      var protoChangeDetector =
          this.changeDetection.createProtoChangeDetector('dummy');
      inheritedProtoView = new ProtoView(current.element, protoChangeDetector,
          this._shadowDomStrategy, this._getParentProtoView(parent));
      if (isPresent(parent)) {
        if (isPresent(parent.inheritedElementBinder.nestedProtoView)) {
          throw new BaseException(
              'Only one nested view per element is allowed');
        }
        parent.inheritedElementBinder.nestedProtoView = inheritedProtoView;
        if (isPresent(parent.variableBindings)) {
          MapWrapper.forEach(parent.variableBindings, (mappedName, varName) {
            inheritedProtoView.bindVariable(varName, mappedName);
          });
        }
      }
    } else if (isPresent(parent)) {
      inheritedProtoView = parent.inheritedProtoView;
    }
    if (isPresent(current.variableBindings)) {
      MapWrapper.forEach(current.variableBindings, (mappedName, varName) {
        MapWrapper.set(inheritedProtoView.protoLocals, mappedName, null);
      });
    }
    current.inheritedProtoView = inheritedProtoView;
  }
  _getParentProtoView(CompileElement parent) {
    return isPresent(parent) ? parent.inheritedProtoView : null;
  }
}
