library angular2.src.core.compiler.pipeline.view_splitter;

import 'package:angular2/src/facade/lang.dart'
    show isBlank, isPresent, BaseException;
import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import 'package:angular2/src/facade/collection.dart'
    show MapWrapper, ListWrapper;
import 'package:angular2/change_detection.dart' show Parser;
import './compile_step.dart' show CompileStep;
import './compile_element.dart' show CompileElement;
import './compile_control.dart' show CompileControl;
import 'package:angular2/src/facade/lang.dart' show StringWrapper;

class ViewSplitter extends CompileStep {
  Parser _parser;
  ViewSplitter(Parser parser) : super() {
    this._parser = parser;
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    var attrs = current.attrs();
    var templateBindings = MapWrapper.get(attrs, 'template');
    var hasTemplateBinding = isPresent(templateBindings);
    MapWrapper.forEach(attrs, (attrValue, attrName) {
      if (StringWrapper.startsWith(attrName, '*')) {
        var key = StringWrapper.substring(attrName, 1);
        if (hasTemplateBinding) {
          throw new BaseException(
              '''Only one template directive per element is allowed: ''' +
                  '''${templateBindings} and ${key} cannot be used simultaneously ''' +
                  '''in ${current.elementDescription}''');
        } else {
          templateBindings =
              (attrValue.length == 0) ? key : key + ' ' + attrValue;
          hasTemplateBinding = true;
        }
      }
    });
    if (isBlank(parent)) {
      current.isViewRoot = true;
    } else {
      if (DOM.isTemplateElement(current.element)) {
        if (!current.isViewRoot) {
          var viewRoot = new CompileElement(DOM.createTemplate(''));
          var currentElement = current.element;
          var viewRootElement = viewRoot.element;
          this._moveChildNodes(
              DOM.content(currentElement), DOM.content(viewRootElement));
          viewRoot.elementDescription = current.elementDescription;
          viewRoot.isViewRoot = true;
          control.addChild(viewRoot);
        }
      } else {
        if (hasTemplateBinding) {
          var newParent = new CompileElement(DOM.createTemplate(''));
          newParent.elementDescription = current.elementDescription;
          current.isViewRoot = true;
          this._parseTemplateBindings(templateBindings, newParent);
          this._addParentElement(current.element, newParent.element);
          control.addParent(newParent);
          DOM.remove(current.element);
        }
      }
    }
  }
  _moveChildNodes(source, target) {
    var next = DOM.firstChild(source);
    while (isPresent(next)) {
      DOM.appendChild(target, next);
      next = DOM.firstChild(source);
    }
  }
  _addParentElement(currentElement, newParentElement) {
    DOM.insertBefore(currentElement, newParentElement);
    DOM.appendChild(newParentElement, currentElement);
  }
  _parseTemplateBindings(
      String templateBindings, CompileElement compileElement) {
    var bindings = this._parser.parseTemplateBindings(
        templateBindings, compileElement.elementDescription);
    for (var i = 0; i < bindings.length; i++) {
      var binding = bindings[i];
      if (binding.keyIsVar) {
        compileElement.addVariableBinding(binding.key, binding.name);
        MapWrapper.set(compileElement.attrs(), binding.key, binding.name);
      } else if (isPresent(binding.expression)) {
        compileElement.addPropertyBinding(binding.key, binding.expression);
        MapWrapper.set(
            compileElement.attrs(), binding.key, binding.expression.source);
      } else {
        DOM.setAttribute(compileElement.element, binding.key, '');
      }
    }
  }
}
