library angular2.src.core.compiler.shadow_dom_emulation.light_dom;

import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper;
import 'package:angular2/src/facade/lang.dart' show isBlank, isPresent;
import '../view.dart' as viewModule;
import './content_tag.dart' show Content;

class DestinationLightDom {}
class _Root {
  var node;
  var viewContainer;
  var content;
  _Root(node, viewContainer, content) {
    this.node = node;
    this.viewContainer = viewContainer;
    this.content = content;
  }
}
class LightDom {
  viewModule.View lightDomView;
  viewModule.View shadowDomView;
  List nodes;
  List<_Root> roots;
  LightDom(
      viewModule.View lightDomView, viewModule.View shadowDomView, element) {
    this.lightDomView = lightDomView;
    this.shadowDomView = shadowDomView;
    this.nodes = DOM.childNodesAsList(element);
    this.roots = null;
  }
  redistribute() {
    var tags = this.contentTags();
    if (tags.length > 0) {
      redistributeNodes(tags, this.expandedDomNodes());
    }
  }
  List<Content> contentTags() {
    return this._collectAllContentTags(this.shadowDomView, []);
  }
  List<Content> _collectAllContentTags(
      viewModule.View view, List<Content> acc) {
    var contentTags = view.contentTags;
    var vcs = view.viewContainers;
    for (var i = 0; i < vcs.length; i++) {
      var vc = vcs[i];
      var contentTag = contentTags[i];
      if (isPresent(contentTag)) {
        ListWrapper.push(acc, contentTag);
      }
      if (isPresent(vc)) {
        ListWrapper.forEach(vc.contentTagContainers(), (view) {
          this._collectAllContentTags(view, acc);
        });
      }
    }
    return acc;
  }
  List expandedDomNodes() {
    var res = [];
    var roots = this._roots();
    for (var i = 0; i < roots.length; ++i) {
      var root = roots[i];
      if (isPresent(root.viewContainer)) {
        res = ListWrapper.concat(res, root.viewContainer.nodes());
      } else if (isPresent(root.content)) {
        res = ListWrapper.concat(res, root.content.nodes());
      } else {
        ListWrapper.push(res, root.node);
      }
    }
    return res;
  }
  _roots() {
    if (isPresent(this.roots)) return this.roots;
    var viewContainers = this.lightDomView.viewContainers;
    var contentTags = this.lightDomView.contentTags;
    this.roots = ListWrapper.map(this.nodes, (n) {
      var foundVc = null;
      var foundContentTag = null;
      for (var i = 0; i < viewContainers.length; i++) {
        var vc = viewContainers[i];
        var contentTag = contentTags[i];
        if (isPresent(vc) && identical(vc.templateElement, n)) {
          foundVc = vc;
        }
        if (isPresent(contentTag) &&
            identical(contentTag.contentStartElement, n)) {
          foundContentTag = contentTag;
        }
      }
      return new _Root(n, foundVc, foundContentTag);
    });
    return this.roots;
  }
}
redistributeNodes(List<Content> contents, List nodes) {
  for (var i = 0; i < contents.length; ++i) {
    var content = contents[i];
    var select = content.select;
    var matchSelector = (n) {
      return DOM.elementMatches(n, select);
    };
    if (identical(select.length, 0)) {
      content.insert(nodes);
      ListWrapper.clear(nodes);
    } else {
      var matchingNodes = ListWrapper.filter(nodes, matchSelector);
      content.insert(matchingNodes);
      ListWrapper.removeAll(nodes, matchingNodes);
    }
  }
}
