library angular2.src.core.compiler.shadow_dom_emulation.shadow_css;

import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper;
import 'package:angular2/src/facade/lang.dart'
    show
        StringWrapper,
        RegExp,
        RegExpWrapper,
        RegExpMatcherWrapper,
        isPresent,
        isBlank,
        BaseException,
        int;

class ShadowCss {
  bool strictStyling;
  ShadowCss() {
    this.strictStyling = true;
  }
  String shimStyle(style, String selector, [String hostSelector = '']) {
    var cssText = DOM.getText(style);
    return this.shimCssText(cssText, selector, hostSelector);
  }
  String shimCssText(String cssText, String selector,
      [String hostSelector = '']) {
    cssText = this._insertDirectives(cssText);
    return this._scopeCssText(cssText, selector, hostSelector);
  }
  String _insertDirectives(String cssText) {
    cssText = this._insertPolyfillDirectivesInCssText(cssText);
    return this._insertPolyfillRulesInCssText(cssText);
  }
  String _insertPolyfillDirectivesInCssText(String cssText) {
    return StringWrapper.replaceAllMapped(cssText, _cssContentNextSelectorRe,
        (m) {
      return m[1] + '{';
    });
  }
  String _insertPolyfillRulesInCssText(String cssText) {
    return StringWrapper.replaceAllMapped(cssText, _cssContentRuleRe, (m) {
      var rule = m[0];
      rule = StringWrapper.replace(rule, m[1], '');
      rule = StringWrapper.replace(rule, m[2], '');
      return m[3] + rule;
    });
  }
  String _scopeCssText(
      String cssText, String scopeSelector, String hostSelector) {
    var unscoped = this._extractUnscopedRulesFromCssText(cssText);
    cssText = this._insertPolyfillHostInCssText(cssText);
    cssText = this._convertColonHost(cssText);
    cssText = this._convertColonHostContext(cssText);
    cssText = this._convertShadowDOMSelectors(cssText);
    if (isPresent(scopeSelector)) {
      _withCssRules(cssText, (rules) {
        cssText = this._scopeRules(rules, scopeSelector, hostSelector);
      });
    }
    cssText = cssText + '\n' + unscoped;
    return cssText.trim();
  }
  String _extractUnscopedRulesFromCssText(String cssText) {
    var r = '';
    var m;
    var matcher = RegExpWrapper.matcher(_cssContentUnscopedRuleRe, cssText);
    while (isPresent(m = RegExpMatcherWrapper.next(matcher))) {
      var rule = m[0];
      rule = StringWrapper.replace(rule, m[2], '');
      rule = StringWrapper.replace(rule, m[1], m[3]);
      r = rule + '\n\n';
    }
    return r;
  }
  String _convertColonHost(String cssText) {
    return this._convertColonRule(
        cssText, _cssColonHostRe, this._colonHostPartReplacer);
  }
  String _convertColonHostContext(String cssText) {
    return this._convertColonRule(
        cssText, _cssColonHostContextRe, this._colonHostContextPartReplacer);
  }
  String _convertColonRule(
      String cssText, RegExp regExp, Function partReplacer) {
    return StringWrapper.replaceAllMapped(cssText, regExp, (m) {
      if (isPresent(m[2])) {
        var parts = m[2].split(',');
        var r = [];
        for (var i = 0; i < parts.length; i++) {
          var p = parts[i];
          if (isBlank(p)) break;
          p = p.trim();
          ListWrapper.push(r, partReplacer(_polyfillHostNoCombinator, p, m[3]));
        }
        return r.join(',');
      } else {
        return _polyfillHostNoCombinator + m[3];
      }
    });
  }
  String _colonHostContextPartReplacer(
      String host, String part, String suffix) {
    if (StringWrapper.contains(part, _polyfillHost)) {
      return this._colonHostPartReplacer(host, part, suffix);
    } else {
      return host + part + suffix + ', ' + part + ' ' + host + suffix;
    }
  }
  String _colonHostPartReplacer(String host, String part, String suffix) {
    return host + StringWrapper.replace(part, _polyfillHost, '') + suffix;
  }
  String _convertShadowDOMSelectors(String cssText) {
    for (var i = 0; i < _shadowDOMSelectorsRe.length; i++) {
      cssText =
          StringWrapper.replaceAll(cssText, _shadowDOMSelectorsRe[i], ' ');
    }
    return cssText;
  }
  String _scopeRules(cssRules, String scopeSelector, String hostSelector) {
    var cssText = '';
    if (isPresent(cssRules)) {
      for (var i = 0; i < cssRules.length; i++) {
        var rule = cssRules[i];
        if (DOM.isStyleRule(rule) || DOM.isPageRule(rule)) {
          cssText += this._scopeSelector(rule.selectorText, scopeSelector,
                  hostSelector, this.strictStyling) +
              ' {\n';
          cssText += this._propertiesFromRule(rule) + '\n}\n\n';
        } else if (DOM.isMediaRule(rule)) {
          cssText += '@media ' + rule.media.mediaText + ' {\n';
          cssText +=
              this._scopeRules(rule.cssRules, scopeSelector, hostSelector);
          cssText += '\n}\n\n';
        } else {
          try {
            if (isPresent(rule.cssText)) {
              cssText += rule.cssText + '\n\n';
            }
          } catch (x) {
            if (DOM.isKeyframesRule(rule) && isPresent(rule.cssRules)) {
              cssText += this._ieSafeCssTextFromKeyFrameRule(rule);
            }
          }
        }
      }
    }
    return cssText;
  }
  String _ieSafeCssTextFromKeyFrameRule(rule) {
    var cssText = '@keyframes ' + rule.name + ' {';
    for (var i = 0; i < rule.cssRules.length; i++) {
      var r = rule.cssRules[i];
      cssText += ' ' + r.keyText + ' {' + r.style.cssText + '}';
    }
    cssText += ' }';
    return cssText;
  }
  String _scopeSelector(
      String selector, String scopeSelector, String hostSelector, bool strict) {
    var r = [];
    var parts = selector.split(',');
    for (var i = 0; i < parts.length; i++) {
      var p = parts[i];
      p = p.trim();
      if (this._selectorNeedsScoping(p, scopeSelector)) {
        p = strict && !StringWrapper.contains(p, _polyfillHostNoCombinator)
            ? this._applyStrictSelectorScope(p, scopeSelector)
            : this._applySelectorScope(p, scopeSelector, hostSelector);
      }
      ListWrapper.push(r, p);
    }
    return r.join(', ');
  }
  bool _selectorNeedsScoping(String selector, String scopeSelector) {
    var re = this._makeScopeMatcher(scopeSelector);
    return !isPresent(RegExpWrapper.firstMatch(re, selector));
  }
  RegExp _makeScopeMatcher(String scopeSelector) {
    var lre = RegExpWrapper.create('\\[');
    var rre = RegExpWrapper.create('\\]');
    scopeSelector = StringWrapper.replaceAll(scopeSelector, lre, '\\[');
    scopeSelector = StringWrapper.replaceAll(scopeSelector, rre, '\\]');
    return RegExpWrapper.create(
        '^(' + scopeSelector + ')' + _selectorReSuffix, 'm');
  }
  String _applySelectorScope(
      String selector, String scopeSelector, String hostSelector) {
    return this._applySimpleSelectorScope(
        selector, scopeSelector, hostSelector);
  }
  String _applySimpleSelectorScope(
      String selector, String scopeSelector, String hostSelector) {
    if (isPresent(RegExpWrapper.firstMatch(_polyfillHostRe, selector))) {
      var replaceBy =
          this.strictStyling ? '''[${hostSelector}]''' : scopeSelector;
      selector =
          StringWrapper.replace(selector, _polyfillHostNoCombinator, replaceBy);
      return StringWrapper.replaceAll(
          selector, _polyfillHostRe, replaceBy + ' ');
    } else {
      return scopeSelector + ' ' + selector;
    }
  }
  String _applyStrictSelectorScope(String selector, String scopeSelector) {
    var isRe = RegExpWrapper.create('\\[is=([^\\]]*)\\]');
    scopeSelector = StringWrapper.replaceAllMapped(scopeSelector, isRe, (m) {
      return m[1];
    });
    var splits = [' ', '>', '+', '~'];
    var scoped = selector;
    var attrName = '[' + scopeSelector + ']';
    for (var i = 0; i < splits.length; i++) {
      var sep = splits[i];
      var parts = scoped.split(sep);
      scoped = ListWrapper.map(parts, (p) {
        var t = StringWrapper.replaceAll(p.trim(), _polyfillHostRe, '');
        if (t.length > 0 &&
            !ListWrapper.contains(splits, t) &&
            !StringWrapper.contains(t, attrName)) {
          var re = RegExpWrapper.create('([^:]*)(:*)(.*)');
          var m = RegExpWrapper.firstMatch(re, t);
          if (isPresent(m)) {
            p = m[1] + attrName + m[2] + m[3];
          }
        }
        return p;
      }).join(sep);
    }
    return scoped;
  }
  String _insertPolyfillHostInCssText(String selector) {
    selector = StringWrapper.replaceAll(
        selector, _colonHostContextRe, _polyfillHostContext);
    selector = StringWrapper.replaceAll(selector, _colonHostRe, _polyfillHost);
    return selector;
  }
  String _propertiesFromRule(rule) {
    var cssText = rule.style.cssText;
    var attrRe = RegExpWrapper.create('[\'"]+|attr');
    if (rule.style.content.length > 0 &&
        !isPresent(RegExpWrapper.firstMatch(attrRe, rule.style.content))) {
      var contentRe = RegExpWrapper.create('content:[^;]*;');
      cssText = StringWrapper.replaceAll(
          cssText, contentRe, 'content: \'' + rule.style.content + '\';');
    }
    return cssText;
  }
}
var _cssContentNextSelectorRe = RegExpWrapper.create(
    'polyfill-next-selector[^}]*content:[\\s]*?[\'"](.*?)[\'"][;\\s]*}([^{]*?){',
    'im');
var _cssContentRuleRe = RegExpWrapper.create(
    '(polyfill-rule)[^}]*(content:[\\s]*[\'"](.*?)[\'"])[;\\s]*[^}]*}', 'im');
var _cssContentUnscopedRuleRe = RegExpWrapper.create(
    '(polyfill-unscoped-rule)[^}]*(content:[\\s]*[\'"](.*?)[\'"])[;\\s]*[^}]*}',
    'im');
var _polyfillHost = '-shadowcsshost';
var _polyfillHostContext = '-shadowcsscontext';
var _parenSuffix = ')(?:\\((' + '(?:\\([^)(]*\\)|[^)(]*)+?' + ')\\))?([^,{]*)';
var _cssColonHostRe =
    RegExpWrapper.create('(' + _polyfillHost + _parenSuffix, 'im');
var _cssColonHostContextRe =
    RegExpWrapper.create('(' + _polyfillHostContext + _parenSuffix, 'im');
var _polyfillHostNoCombinator = _polyfillHost + '-no-combinator';
var _shadowDOMSelectorsRe = [
  RegExpWrapper.create('>>>'),
  RegExpWrapper.create('::shadow'),
  RegExpWrapper.create('::content'),
  RegExpWrapper.create('/deep/'),
  RegExpWrapper.create('/shadow-deep/'),
  RegExpWrapper.create('/shadow/')
];
var _selectorReSuffix = '([>\\s~+\[.,{:][\\s\\S]*)?\$';
var _polyfillHostRe = RegExpWrapper.create(_polyfillHost, 'im');
var _colonHostRe = RegExpWrapper.create(':host', 'im');
var _colonHostContextRe = RegExpWrapper.create(':host-context', 'im');
_cssToRules(String cssText) {
  return DOM.cssToRules(cssText);
}
_withCssRules(String cssText, Function callback) {
  if (isBlank(callback)) return;
  var rules = _cssToRules(cssText);
  callback(rules);
}
