library angular2.src.core.compiler.shadow_dom_strategy;

import 'package:angular2/di.dart' show Injectable;
import 'package:angular2/src/facade/lang.dart'
    show Type, isBlank, isPresent, int, StringWrapper, assertionsEnabled;
import 'package:angular2/src/facade/collection.dart'
    show List, ListWrapper, MapWrapper, Map;
import 'package:angular2/src/facade/async.dart' show PromiseWrapper;
import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import './view.dart' as viewModule;
import './shadow_dom_emulation/light_dom.dart' show LightDom;
import './shadow_dom_emulation/shadow_css.dart' show ShadowCss;
import './style_inliner.dart' show StyleInliner;
import './style_url_resolver.dart' show StyleUrlResolver;
import './directive_metadata.dart' show DirectiveMetadata;
import './pipeline/compile_step.dart' as NS;
import './pipeline/compile_element.dart' show CompileElement;
import './pipeline/compile_control.dart' show CompileControl;

var _EMPTY_STEP;
_emptyStep() {
  if (isBlank(_EMPTY_STEP)) {
    _EMPTY_STEP = new _EmptyCompileStep();
  }
  return _EMPTY_STEP;
}
class ShadowDomStrategy {
  attachTemplate(el, viewModule.View view) {}
  LightDom constructLightDom(
      viewModule.View lightDomView, viewModule.View shadowDomView, el) {
    return null;
  }
  NS.CompileStep getStyleCompileStep(
      DirectiveMetadata cmpMetadata, String templateUrl) {
    return _emptyStep();
  }
  NS.CompileStep getTemplateCompileStep(DirectiveMetadata cmpMetadata) {
    return _emptyStep();
  }
  shimAppElement(DirectiveMetadata cmpMetadata, element) {}
}
@Injectable()
class EmulatedUnscopedShadowDomStrategy extends ShadowDomStrategy {
  StyleUrlResolver _styleUrlResolver;
  var _styleHost;
  EmulatedUnscopedShadowDomStrategy(
      StyleUrlResolver styleUrlResolver, styleHost)
      : super() {
    this._styleUrlResolver = styleUrlResolver;
    this._styleHost = styleHost;
  }
  attachTemplate(el, viewModule.View view) {
    DOM.clearNodes(el);
    _moveViewNodesIntoParent(el, view);
  }
  LightDom constructLightDom(
      viewModule.View lightDomView, viewModule.View shadowDomView, el) {
    return new LightDom(lightDomView, shadowDomView, el);
  }
  NS.CompileStep getStyleCompileStep(
      DirectiveMetadata cmpMetadata, String templateUrl) {
    return new _EmulatedUnscopedCssStep(
        cmpMetadata, templateUrl, this._styleUrlResolver, this._styleHost);
  }
  NS.CompileStep getTemplateCompileStep(DirectiveMetadata cmpMetadata) {
    return new _BaseEmulatedShadowDomStep();
  }
}
@Injectable()
class EmulatedScopedShadowDomStrategy
    extends EmulatedUnscopedShadowDomStrategy {
  StyleInliner _styleInliner;
  EmulatedScopedShadowDomStrategy(
      StyleInliner styleInliner, StyleUrlResolver styleUrlResolver, styleHost)
      : super(styleUrlResolver, styleHost) {
    this._styleInliner = styleInliner;
  }
  NS.CompileStep getStyleCompileStep(
      DirectiveMetadata cmpMetadata, String templateUrl) {
    return new _EmulatedScopedCssStep(cmpMetadata, templateUrl,
        this._styleInliner, this._styleUrlResolver, this._styleHost);
  }
  NS.CompileStep getTemplateCompileStep(DirectiveMetadata cmpMetadata) {
    return new _ShimShadowDomStep(cmpMetadata);
  }
  shimAppElement(DirectiveMetadata cmpMetadata, element) {
    var cmpType = cmpMetadata.type;
    var hostAttribute = _getHostAttribute(_getComponentId(cmpType));
    DOM.setAttribute(element, hostAttribute, '');
  }
}
@Injectable()
class NativeShadowDomStrategy extends ShadowDomStrategy {
  StyleUrlResolver _styleUrlResolver;
  NativeShadowDomStrategy(StyleUrlResolver styleUrlResolver) : super() {
    this._styleUrlResolver = styleUrlResolver;
  }
  attachTemplate(el, viewModule.View view) {
    _moveViewNodesIntoParent(DOM.createShadowRoot(el), view);
  }
  NS.CompileStep getStyleCompileStep(
      DirectiveMetadata cmpMetadata, String templateUrl) {
    return new _NativeCssStep(templateUrl, this._styleUrlResolver);
  }
}
class _BaseEmulatedShadowDomStep extends NS.CompileStep {
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    if (current.ignoreBindings) {
      return;
    }
    var nodeName = DOM.nodeName(current.element);
    if (StringWrapper.equals(nodeName.toUpperCase(), 'CONTENT')) {
      var attrs = current.attrs();
      var selector = MapWrapper.get(attrs, 'select');
      current.contentTagSelector = isPresent(selector) ? selector : '';
      var contentStart = DOM.createScriptTag('type', 'ng/contentStart');
      if (assertionsEnabled()) {
        DOM.setAttribute(contentStart, 'select', current.contentTagSelector);
      }
      var contentEnd = DOM.createScriptTag('type', 'ng/contentEnd');
      DOM.insertBefore(current.element, contentStart);
      DOM.insertBefore(current.element, contentEnd);
      DOM.remove(current.element);
      current.element = contentStart;
    }
  }
}
class _EmptyCompileStep extends NS.CompileStep {
  process(
      CompileElement parent, CompileElement current, CompileControl control) {}
}
class _ShimShadowDomStep extends _BaseEmulatedShadowDomStep {
  String _contentAttribute;
  _ShimShadowDomStep(DirectiveMetadata cmpMetadata) : super() {
    var id = _getComponentId(cmpMetadata.type);
    this._contentAttribute = _getContentAttribute(id);
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    super.process(parent, current, control);
    if (current.ignoreBindings) {
      return;
    }
    DOM.setAttribute(current.element, this._contentAttribute, '');
    var host = current.componentDirective;
    if (isPresent(host)) {
      var hostId = _getComponentId(host.type);
      var hostAttribute = _getHostAttribute(hostId);
      DOM.setAttribute(current.element, hostAttribute, '');
    }
  }
}
class _EmulatedUnscopedCssStep extends NS.CompileStep {
  String _templateUrl;
  StyleUrlResolver _styleUrlResolver;
  var _styleHost;
  _EmulatedUnscopedCssStep(DirectiveMetadata cmpMetadata, String templateUrl,
      StyleUrlResolver styleUrlResolver, styleHost)
      : super() {
    this._templateUrl = templateUrl;
    this._styleUrlResolver = styleUrlResolver;
    this._styleHost = styleHost;
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    var styleEl = current.element;
    var cssText = DOM.getText(styleEl);
    cssText = this._styleUrlResolver.resolveUrls(cssText, this._templateUrl);
    DOM.setText(styleEl, cssText);
    DOM.remove(styleEl);
    if (!MapWrapper.contains(_sharedStyleTexts, cssText)) {
      MapWrapper.set(_sharedStyleTexts, cssText, true);
      _insertStyleElement(this._styleHost, styleEl);
    }
  }
}
class _EmulatedScopedCssStep extends NS.CompileStep {
  String _templateUrl;
  Type _component;
  StyleInliner _styleInliner;
  StyleUrlResolver _styleUrlResolver;
  var _styleHost;
  _EmulatedScopedCssStep(DirectiveMetadata cmpMetadata, String templateUrl,
      StyleInliner styleInliner, StyleUrlResolver styleUrlResolver, styleHost)
      : super() {
    this._templateUrl = templateUrl;
    this._component = cmpMetadata.type;
    this._styleInliner = styleInliner;
    this._styleUrlResolver = styleUrlResolver;
    this._styleHost = styleHost;
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    var styleEl = current.element;
    var cssText = DOM.getText(styleEl);
    cssText = this._styleUrlResolver.resolveUrls(cssText, this._templateUrl);
    var css = this._styleInliner.inlineImports(cssText, this._templateUrl);
    if (PromiseWrapper.isPromise(css)) {
      DOM.setText(styleEl, '');
      ListWrapper.push(parent.inheritedProtoView.stylePromises, css);
      return css.then((css) {
        css = _shimCssForComponent(css, this._component);
        DOM.setText(styleEl, css);
      });
    } else {
      css = _shimCssForComponent(css, this._component);
      DOM.setText(styleEl, css);
    }
    DOM.remove(styleEl);
    _insertStyleElement(this._styleHost, styleEl);
  }
}
class _NativeCssStep extends NS.CompileStep {
  StyleUrlResolver _styleUrlResolver;
  String _templateUrl;
  _NativeCssStep(String templateUrl, StyleUrlResolver styleUrlResover)
      : super() {
    this._styleUrlResolver = styleUrlResover;
    this._templateUrl = templateUrl;
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    var styleEl = current.element;
    var cssText = DOM.getText(styleEl);
    cssText = this._styleUrlResolver.resolveUrls(cssText, this._templateUrl);
    DOM.setText(styleEl, cssText);
  }
}
_moveViewNodesIntoParent(parent, view) {
  for (var i = 0; i < view.nodes.length; ++i) {
    DOM.appendChild(parent, view.nodes[i]);
  }
}
Map<Type, int> _componentUIDs = MapWrapper.create();
int _nextComponentUID = 0;
Map<String, bool> _sharedStyleTexts = MapWrapper.create();
var _lastInsertedStyleEl;
_getComponentId(Type component) {
  var id = MapWrapper.get(_componentUIDs, component);
  if (isBlank(id)) {
    id = _nextComponentUID++;
    MapWrapper.set(_componentUIDs, component, id);
  }
  return id;
}
_insertStyleElement(host, styleEl) {
  if (isBlank(_lastInsertedStyleEl)) {
    var firstChild = DOM.firstChild(host);
    if (isPresent(firstChild)) {
      DOM.insertBefore(firstChild, styleEl);
    } else {
      DOM.appendChild(host, styleEl);
    }
  } else {
    DOM.insertAfter(_lastInsertedStyleEl, styleEl);
  }
  _lastInsertedStyleEl = styleEl;
}
_getHostAttribute(int id) {
  return '''_nghost-${id}''';
}
_getContentAttribute(int id) {
  return '''_ngcontent-${id}''';
}
String _shimCssForComponent(String cssText, Type component) {
  var id = _getComponentId(component);
  var shadowCss = new ShadowCss();
  return shadowCss.shimCssText(
      cssText, _getContentAttribute(id), _getHostAttribute(id));
}
resetShadowDomCache() {
  MapWrapper.clear(_componentUIDs);
  _nextComponentUID = 0;
  MapWrapper.clear(_sharedStyleTexts);
  _lastInsertedStyleEl = null;
}
