library angular2.src.core.compiler.style_inliner;

import 'package:angular2/di.dart' show Injectable;
import 'package:angular2/src/core/compiler/xhr/xhr.dart' show XHR;
import 'package:angular2/src/core/compiler/style_url_resolver.dart'
    show StyleUrlResolver;
import 'package:angular2/src/core/compiler/url_resolver.dart' show UrlResolver;
import 'package:angular2/src/facade/collection.dart' show ListWrapper;
import 'package:angular2/src/facade/lang.dart'
    show
        isBlank,
        isPresent,
        RegExp,
        RegExpWrapper,
        StringWrapper,
        normalizeBlank;
import 'package:angular2/src/facade/async.dart' show Future, PromiseWrapper;

@Injectable()
class StyleInliner {
  XHR _xhr;
  UrlResolver _urlResolver;
  StyleUrlResolver _styleUrlResolver;
  StyleInliner(
      XHR xhr, StyleUrlResolver styleUrlResolver, UrlResolver urlResolver) {
    this._xhr = xhr;
    this._urlResolver = urlResolver;
    this._styleUrlResolver = styleUrlResolver;
  }
  inlineImports(String cssText, String baseUrl) {
    return this._inlineImports(cssText, baseUrl, []);
  }
  _inlineImports(String cssText, String baseUrl, List<String> inlinedUrls) {
    var partIndex = 0;
    var parts = StringWrapper.split(cssText, _importRe);
    if (identical(parts.length, 1)) {
      return cssText;
    }
    var promises = [];
    while (partIndex < parts.length - 1) {
      var prefix = parts[partIndex];
      var rule = parts[partIndex + 1];
      var url = _extractUrl(rule);
      if (isPresent(url)) {
        url = this._urlResolver.resolve(baseUrl, url);
      }
      var mediaQuery = _extractMediaQuery(rule);
      var promise;
      if (isBlank(url)) {
        promise = PromiseWrapper
            .resolve('''/* Invalid import rule: \"@import ${rule};\" */''');
      } else if (ListWrapper.contains(inlinedUrls, url)) {
        promise = PromiseWrapper.resolve(prefix);
      } else {
        ListWrapper.push(inlinedUrls, url);
        promise = PromiseWrapper.then(this._xhr.get(url), (css) {
          css = this._inlineImports(css, url, inlinedUrls);
          if (PromiseWrapper.isPromise(css)) {
            return css.then((css) {
              return prefix +
                  this._transformImportedCss(css, mediaQuery, url) +
                  '\n';
            });
          } else {
            return prefix +
                this._transformImportedCss(css, mediaQuery, url) +
                '\n';
          }
        }, (error) {
          return '''/* failed to import ${url} */\n''';
        });
      }
      ListWrapper.push(promises, promise);
      partIndex += 2;
    }
    return PromiseWrapper.all(promises).then((cssParts) {
      var cssText = cssParts.join('');
      if (partIndex < parts.length) {
        cssText += parts[partIndex];
      }
      return cssText;
    });
  }
  String _transformImportedCss(String css, String mediaQuery, String url) {
    css = this._styleUrlResolver.resolveUrls(css, url);
    return _wrapInMediaRule(css, mediaQuery);
  }
}
String _extractUrl(String importRule) {
  var match = RegExpWrapper.firstMatch(_urlRe, importRule);
  if (isBlank(match)) return null;
  return isPresent(match[1]) ? match[1] : match[2];
}
String _extractMediaQuery(String importRule) {
  var match = RegExpWrapper.firstMatch(_mediaQueryRe, importRule);
  if (isBlank(match)) return null;
  var mediaQuery = match[1].trim();
  return (mediaQuery.length > 0) ? mediaQuery : null;
}
String _wrapInMediaRule(String css, String query) {
  return (isBlank(query)) ? css : '''@media ${query} {\n${css}\n}''';
}
var _importRe = RegExpWrapper.create('@import\\s+([^;]+);');
var _urlRe = RegExpWrapper
    .create('url\\(\\s*?[\'"]?([^\'")]+)[\'"]?|' + '[\'"]([^\'")]+)[\'"]');
var _mediaQueryRe = RegExpWrapper.create('[\'"][^\'"]+[\'"]\\s*\\)?\\s*(.*)');
