library angular2.src.core.compiler.template_loader;

import 'package:angular2/di.dart' show Injectable;
import 'package:angular2/src/facade/lang.dart'
    show isBlank, isPresent, BaseException, stringify;
import 'package:angular2/src/facade/collection.dart'
    show Map, MapWrapper, StringMapWrapper, Map;
import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import './xhr/xhr.dart' show XHR;
import 'package:angular2/src/core/annotations/template.dart' show Template;
import './url_resolver.dart' show UrlResolver;

@Injectable()
class TemplateLoader {
  XHR _xhr;
  Map _htmlCache;
  Map<Type, String> _baseUrls;
  Map<Type, String> _urlCache;
  UrlResolver _urlResolver;
  TemplateLoader(XHR xhr, UrlResolver urlResolver) {
    this._xhr = xhr;
    this._urlResolver = urlResolver;
    this._htmlCache = StringMapWrapper.create();
    this._baseUrls = MapWrapper.create();
    this._urlCache = MapWrapper.create();
  }
  load(Template template) {
    if (isPresent(template.inline)) {
      return DOM.createTemplate(template.inline);
    }
    if (isPresent(template.url)) {
      var url = this.getTemplateUrl(template);
      var promise = StringMapWrapper.get(this._htmlCache, url);
      if (isBlank(promise)) {
        promise = this._xhr.get(url).then((html) {
          var template = DOM.createTemplate(html);
          return template;
        });
        StringMapWrapper.set(this._htmlCache, url, promise);
      }
      return promise;
    }
    throw new BaseException(
        'Templates should have either their url or inline property set');
  }
  setBaseUrl(Template template, String baseUrl) {
    MapWrapper.set(this._baseUrls, template, baseUrl);
    MapWrapper.delete(this._urlCache, template);
  }
  getTemplateUrl(Template template) {
    if (!MapWrapper.contains(this._urlCache, template)) {
      var baseUrl = MapWrapper.get(this._baseUrls, template);
      if (isBlank(baseUrl)) {
        throw new BaseException('The template base URL is not set');
      }
      var templateUrl;
      if (isPresent(template.url)) {
        templateUrl = this._urlResolver.resolve(baseUrl, template.url);
      } else {
        templateUrl = baseUrl;
      }
      MapWrapper.set(this._urlCache, template, templateUrl);
    }
    return MapWrapper.get(this._urlCache, template);
  }
}
