library angular2.src.core.compiler.template_resolver;

import 'package:angular2/di.dart' show Injectable;
import 'package:angular2/src/core/annotations/template.dart' show Template;
import 'package:angular2/src/facade/lang.dart'
    show Type, stringify, isBlank, BaseException;
import 'package:angular2/src/facade/collection.dart'
    show Map, MapWrapper, List, ListWrapper;
import 'package:angular2/src/reflection/reflection.dart' show reflector;

@Injectable()
class TemplateResolver {
  Map _cache;
  TemplateResolver() {
    this._cache = MapWrapper.create();
  }
  Template resolve(Type component) {
    var template = MapWrapper.get(this._cache, component);
    if (isBlank(template)) {
      template = this._resolve(component);
      MapWrapper.set(this._cache, component, template);
    }
    return template;
  }
  _resolve(Type component) {
    var annotations = reflector.annotations(component);
    for (var i = 0; i < annotations.length; i++) {
      var annotation = annotations[i];
      if (annotation is Template) {
        return annotation;
      }
    }
    throw new BaseException(
        '''No template found for ${stringify(component)}''');
  }
}
