library angular2.src.core.compiler.view;

import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import 'package:angular2/src/facade/async.dart' show Future;
import 'package:angular2/src/facade/collection.dart'
    show ListWrapper, MapWrapper, Map, StringMapWrapper, List;
import 'package:angular2/change_detection.dart'
    show
        AST,
        Locals,
        ChangeDispatcher,
        ProtoChangeDetector,
        ChangeDetector,
        ChangeRecord,
        BindingRecord,
        uninitialized;
import './element_injector.dart'
    show ProtoElementInjector, ElementInjector, PreBuiltObjects;
import './binding_propagation_config.dart' show BindingPropagationConfig;
import './element_binder.dart' show ElementBinder;
import './directive_metadata.dart' show DirectiveMetadata;
import 'package:angular2/src/reflection/types.dart' show SetterFn;
import 'package:angular2/src/facade/lang.dart'
    show IMPLEMENTS, int, isPresent, isBlank, BaseException;
import 'package:angular2/di.dart' show Injector;
import 'package:angular2/src/core/dom/element.dart' show NgElement;
import './view_container.dart' show ViewContainer;
import './shadow_dom_emulation/light_dom.dart' show LightDom;
import './shadow_dom_emulation/content_tag.dart' show Content;
import './shadow_dom_strategy.dart' show ShadowDomStrategy;
import './view_pool.dart' show ViewPool;
import 'package:angular2/src/core/events/event_manager.dart' show EventManager;

const NG_BINDING_CLASS = 'ng-binding';
const NG_BINDING_CLASS_SELECTOR = '.ng-binding';
var VIEW_POOL_CAPACITY = 10000;
var VIEW_POOL_PREFILL = 0;
@IMPLEMENTS(ChangeDispatcher)
class View {
  List<ElementInjector> rootElementInjectors;
  List<ElementInjector> elementInjectors;
  List bindElements;
  List textNodes;
  ChangeDetector changeDetector;
  List nodes;
  List<View> componentChildViews;
  List<ViewContainer> viewContainers;
  List<Content> contentTags;
  List<PreBuiltObjects> preBuiltObjects;
  List<LightDom> lightDoms;
  ProtoView proto;
  dynamic context;
  Locals locals;
  View(ProtoView proto, List nodes, Map protoLocals) {
    this.proto = proto;
    this.nodes = nodes;
    this.changeDetector = null;
    this.elementInjectors = null;
    this.rootElementInjectors = null;
    this.textNodes = null;
    this.bindElements = null;
    this.componentChildViews = null;
    this.viewContainers = null;
    this.contentTags = null;
    this.preBuiltObjects = null;
    this.lightDoms = null;
    this.context = null;
    this.locals = new Locals(null, MapWrapper.clone(protoLocals));
  }
  init(ChangeDetector changeDetector, List elementInjectors,
      List rootElementInjectors, List textNodes, List bindElements,
      List viewContainers, List contentTags, List preBuiltObjects,
      List componentChildViews, List<LightDom> lightDoms) {
    this.changeDetector = changeDetector;
    this.elementInjectors = elementInjectors;
    this.rootElementInjectors = rootElementInjectors;
    this.textNodes = textNodes;
    this.bindElements = bindElements;
    this.viewContainers = viewContainers;
    this.contentTags = contentTags;
    this.preBuiltObjects = preBuiltObjects;
    this.componentChildViews = componentChildViews;
    this.lightDoms = lightDoms;
  }
  setLocal(String contextName, value) {
    if (!this.hydrated()) throw new BaseException(
        'Cannot set locals on dehydrated view.');
    if (!MapWrapper.contains(this.proto.variableBindings, contextName)) {
      return;
    }
    var templateName = MapWrapper.get(this.proto.variableBindings, contextName);
    this.locals.set(templateName, value);
  }
  hydrated() {
    return isPresent(this.context);
  }
  _hydrateContext(newContext, locals) {
    this.context = newContext;
    this.locals.parent = locals;
    this.changeDetector.hydrate(this.context, this.locals);
  }
  _dehydrateContext() {
    if (isPresent(this.locals)) {
      this.locals.clearValues();
    }
    this.context = null;
    this.changeDetector.dehydrate();
  }
  hydrate(Injector appInjector, ElementInjector hostElementInjector,
      LightDom hostLightDom, Object context, Locals locals) {
    if (this
        .hydrated()) throw new BaseException('The view is already hydrated.');
    this._hydrateContext(context, locals);
    for (var i = 0; i < this.viewContainers.length; i++) {
      var vc = this.viewContainers[i];
      if (isPresent(vc)) {
        vc.hydrate(appInjector, hostElementInjector, hostLightDom);
      }
    }
    var binders = this.proto.elementBinders;
    var componentChildViewIndex = 0;
    for (var i = 0; i < binders.length; ++i) {
      var componentDirective = binders[i].componentDirective;
      var shadowDomAppInjector = null;
      if (isPresent(componentDirective)) {
        var services = componentDirective.annotation.services;
        if (isPresent(services)) shadowDomAppInjector =
            appInjector.createChild(services);
        else {
          shadowDomAppInjector = appInjector;
        }
      } else {
        shadowDomAppInjector = null;
      }
      var elementInjector = this.elementInjectors[i];
      if (isPresent(elementInjector)) {
        elementInjector.instantiateDirectives(
            appInjector, shadowDomAppInjector, this.preBuiltObjects[i]);
        var exportImplicitName = elementInjector.getExportImplicitName();
        if (elementInjector.isExportingComponent()) {
          this.locals.set(exportImplicitName, elementInjector.getComponent());
        } else if (elementInjector.isExportingElement()) {
          this.locals.set(
              exportImplicitName, elementInjector.getNgElement().domElement);
        }
      }
      if (isPresent(binders[i].nestedProtoView) &&
          isPresent(componentDirective)) {
        this.componentChildViews[componentChildViewIndex++].hydrate(
            shadowDomAppInjector, elementInjector, this.lightDoms[i],
            elementInjector.getComponent(), null);
      }
    }
    for (var i = 0; i < this.lightDoms.length; ++i) {
      var lightDom = this.lightDoms[i];
      if (isPresent(lightDom)) {
        lightDom.redistribute();
      }
    }
  }
  dehydrate() {
    for (var i = 0; i < this.componentChildViews.length; i++) {
      this.componentChildViews[i].dehydrate();
    }
    for (var i = 0; i < this.elementInjectors.length; i++) {
      if (isPresent(this.elementInjectors[i])) {
        this.elementInjectors[i].clearDirectives();
      }
    }
    if (isPresent(this.viewContainers)) {
      for (var i = 0; i < this.viewContainers.length; i++) {
        var vc = this.viewContainers[i];
        if (isPresent(vc)) {
          vc.dehydrate();
        }
      }
    }
    this._dehydrateContext();
  }
  triggerEventHandlers(String eventName, eventObj, int binderIndex) {
    var handlers = this.proto.eventHandlers[binderIndex];
    if (isBlank(handlers)) return;
    var handler = StringMapWrapper.get(handlers, eventName);
    if (isBlank(handler)) return;
    handler(eventObj, this);
  }
  onRecordChange(directiveMemento, List records) {
    this._invokeMementos(records);
    if (directiveMemento is DirectiveMemento) {
      this._notifyDirectiveAboutChanges(directiveMemento, records);
    }
  }
  _invokeMementos(List records) {
    for (var i = 0; i < records.length; ++i) {
      this._invokeMementoFor(records[i]);
    }
  }
  _notifyDirectiveAboutChanges(directiveMemento, List records) {
    var dir = directiveMemento.directive(this.elementInjectors);
    var binding = directiveMemento.directiveBinding(this.elementInjectors);
    if (binding.callOnChange) {
      dir.onChange(this._collectChanges(records));
    }
  }
  _invokeMementoFor(ChangeRecord record) {
    var memento = record.bindingMemento;
    if (memento is DirectiveBindingMemento) {
      DirectiveBindingMemento directiveMemento = memento;
      directiveMemento.invoke(record, this.elementInjectors);
    } else if (memento is ElementBindingMemento) {
      ElementBindingMemento elementMemento = memento;
      elementMemento.invoke(record, this.bindElements);
    } else {
      num textNodeIndex = memento;
      DOM.setText(this.textNodes[textNodeIndex], record.currentValue);
    }
  }
  _collectChanges(List records) {
    var changes = StringMapWrapper.create();
    for (var i = 0; i < records.length; ++i) {
      var record = records[i];
      var propertyUpdate =
          new PropertyUpdate(record.currentValue, record.previousValue);
      StringMapWrapper.set(
          changes, record.bindingMemento._setterName, propertyUpdate);
    }
    return changes;
  }
}
class ProtoView {
  var element;
  List<ElementBinder> elementBinders;
  ProtoChangeDetector protoChangeDetector;
  Map variableBindings;
  Map protoLocals;
  int textNodesWithBindingCount;
  int elementsWithBindingCount;
  bool instantiateInPlace;
  int rootBindingOffset;
  bool isTemplateElement;
  ShadowDomStrategy shadowDomStrategy;
  ViewPool _viewPool;
  List<Future> stylePromises;
  List eventHandlers;
  List bindingRecords;
  ProtoView parentProtoView;
  List _variableBindings;
  ProtoView(template, ProtoChangeDetector protoChangeDetector,
      ShadowDomStrategy shadowDomStrategy, [ProtoView parentProtoView = null]) {
    this.element = template;
    this.elementBinders = [];
    this.variableBindings = MapWrapper.create();
    this.protoLocals = MapWrapper.create();
    this.protoChangeDetector = protoChangeDetector;
    this.parentProtoView = parentProtoView;
    this.textNodesWithBindingCount = 0;
    this.elementsWithBindingCount = 0;
    this.instantiateInPlace = false;
    this.rootBindingOffset = (isPresent(this.element) &&
        DOM.hasClass(this.element, NG_BINDING_CLASS)) ? 1 : 0;
    this.isTemplateElement = DOM.isTemplateElement(this.element);
    this.shadowDomStrategy = shadowDomStrategy;
    this._viewPool = new ViewPool(VIEW_POOL_CAPACITY);
    this.stylePromises = [];
    this.eventHandlers = [];
    this.bindingRecords = [];
    this._variableBindings = null;
  }
  View instantiate(
      ElementInjector hostElementInjector, EventManager eventManager) {
    if (this._viewPool.length() == 0) this._preFillPool(
        hostElementInjector, eventManager);
    var view = this._viewPool.pop();
    return isPresent(view)
        ? view
        : this._instantiate(hostElementInjector, eventManager);
  }
  _preFillPool(ElementInjector hostElementInjector, EventManager eventManager) {
    for (var i = 0; i < VIEW_POOL_PREFILL; i++) {
      this._viewPool.push(this._instantiate(hostElementInjector, eventManager));
    }
  }
  _getVariableBindings() {
    if (isPresent(this._variableBindings)) {
      return this._variableBindings;
    }
    this._variableBindings = isPresent(this.parentProtoView)
        ? ListWrapper.clone(this.parentProtoView._getVariableBindings())
        : [];
    MapWrapper.forEach(this.protoLocals, (v, local) {
      ListWrapper.push(this._variableBindings, local);
    });
    return this._variableBindings;
  }
  View _instantiate(
      ElementInjector hostElementInjector, EventManager eventManager) {
    var rootElementClone = this.instantiateInPlace
        ? this.element
        : DOM.importIntoDoc(this.element);
    var elementsWithBindingsDynamic;
    if (this.isTemplateElement) {
      elementsWithBindingsDynamic = DOM.querySelectorAll(
          DOM.content(rootElementClone), NG_BINDING_CLASS_SELECTOR);
    } else {
      elementsWithBindingsDynamic =
          DOM.getElementsByClassName(rootElementClone, NG_BINDING_CLASS);
    }
    var elementsWithBindings =
        ListWrapper.createFixedSize(elementsWithBindingsDynamic.length);
    for (var binderIdx = 0;
        binderIdx < elementsWithBindingsDynamic.length;
        ++binderIdx) {
      elementsWithBindings[binderIdx] = elementsWithBindingsDynamic[binderIdx];
    }
    var viewNodes;
    if (this.isTemplateElement) {
      var childNode = DOM.firstChild(DOM.content(rootElementClone));
      viewNodes = [];
      while (childNode != null) {
        ListWrapper.push(viewNodes, childNode);
        childNode = DOM.nextSibling(childNode);
      }
    } else {
      viewNodes = [rootElementClone];
    }
    var view = new View(this, viewNodes, this.protoLocals);
    var changeDetector = this.protoChangeDetector.instantiate(
        view, this.bindingRecords, this._getVariableBindings());
    var binders = this.elementBinders;
    var elementInjectors = ListWrapper.createFixedSize(binders.length);
    var eventHandlers = ListWrapper.createFixedSize(binders.length);
    var rootElementInjectors = [];
    var textNodes = [];
    var elementsWithPropertyBindings = [];
    var preBuiltObjects = ListWrapper.createFixedSize(binders.length);
    var viewContainers = ListWrapper.createFixedSize(binders.length);
    var contentTags = ListWrapper.createFixedSize(binders.length);
    var componentChildViews = [];
    var lightDoms = ListWrapper.createFixedSize(binders.length);
    for (var binderIdx = 0; binderIdx < binders.length; binderIdx++) {
      var binder = binders[binderIdx];
      var element;
      if (identical(binderIdx, 0) && identical(this.rootBindingOffset, 1)) {
        element = rootElementClone;
      } else {
        element = elementsWithBindings[binderIdx - this.rootBindingOffset];
      }
      var elementInjector = null;
      var protoElementInjector = binder.protoElementInjector;
      if (isPresent(protoElementInjector)) {
        if (isPresent(protoElementInjector.parent)) {
          var parentElementInjector = elementInjectors[
              protoElementInjector.parent.index];
          elementInjector =
              protoElementInjector.instantiate(parentElementInjector, null);
        } else {
          elementInjector =
              protoElementInjector.instantiate(null, hostElementInjector);
          ListWrapper.push(rootElementInjectors, elementInjector);
        }
      }
      elementInjectors[binderIdx] = elementInjector;
      if (binder.hasElementPropertyBindings) {
        ListWrapper.push(elementsWithPropertyBindings, element);
      }
      var textNodeIndices = binder.textNodeIndices;
      if (isPresent(textNodeIndices)) {
        var childNode = DOM.firstChild(DOM.templateAwareRoot(element));
        for (var j = 0, k = 0; j < textNodeIndices.length; j++) {
          for (var index = textNodeIndices[j]; k < index; k++) {
            childNode = DOM.nextSibling(childNode);
          }
          ListWrapper.push(textNodes, childNode);
        }
      }
      var lightDom = null;
      var bindingPropagationConfig = null;
      if (isPresent(binder.nestedProtoView) &&
          isPresent(binder.componentDirective)) {
        var strategy = this.shadowDomStrategy;
        var childView =
            binder.nestedProtoView.instantiate(elementInjector, eventManager);
        changeDetector.addChild(childView.changeDetector);
        lightDom = strategy.constructLightDom(view, childView, element);
        strategy.attachTemplate(element, childView);
        bindingPropagationConfig = new BindingPropagationConfig(changeDetector);
        ListWrapper.push(componentChildViews, childView);
      }
      lightDoms[binderIdx] = lightDom;
      var destLightDom = null;
      if (isPresent(binder.parent) && identical(binder.distanceToParent, 1)) {
        destLightDom = lightDoms[binder.parent.index];
      }
      var viewContainer = null;
      if (isPresent(binder.viewportDirective)) {
        viewContainer = new ViewContainer(view, element, binder.nestedProtoView,
            elementInjector, eventManager, destLightDom);
      }
      viewContainers[binderIdx] = viewContainer;
      var contentTag = null;
      if (isPresent(binder.contentTagSelector)) {
        contentTag =
            new Content(destLightDom, element, binder.contentTagSelector);
      }
      contentTags[binderIdx] = contentTag;
      if (isPresent(elementInjector)) {
        preBuiltObjects[binderIdx] = new PreBuiltObjects(view,
            new NgElement(element), viewContainer, bindingPropagationConfig);
      }
      if (isPresent(binder.events)) {
        eventHandlers[binderIdx] = StringMapWrapper.create();
        StringMapWrapper.forEach(binder.events, (eventMap, eventName) {
          var handler = ProtoView.buildEventHandler(eventMap, binderIdx);
          StringMapWrapper.set(eventHandlers[binderIdx], eventName, handler);
          if (isBlank(elementInjector) ||
              !elementInjector.hasEventEmitter(eventName)) {
            eventManager.addEventListener(element, eventName, (event) {
              handler(event, view);
            });
          }
        });
      }
    }
    this.eventHandlers = eventHandlers;
    view.init(changeDetector, elementInjectors, rootElementInjectors, textNodes,
        elementsWithPropertyBindings, viewContainers, contentTags,
        preBuiltObjects, componentChildViews, lightDoms);
    return view;
  }
  returnToPool(View view) {
    this._viewPool.push(view);
  }
  static buildEventHandler(Map eventMap, int injectorIdx) {
    var locals = MapWrapper.create();
    return (event, view) {
      if (view.hydrated()) {
        MapWrapper.set(locals, '\$event', event);
        MapWrapper.forEach(eventMap, (expr, directiveIndex) {
          var context;
          if (identical(directiveIndex, -1)) {
            context = view.context;
          } else {
            context = view.elementInjectors[injectorIdx]
                .getDirectiveAtIndex(directiveIndex);
          }
          expr.eval(context, new Locals(view.locals, locals));
        });
      }
    };
  }
  bindVariable(String contextName, String templateName) {
    MapWrapper.set(this.variableBindings, contextName, templateName);
    MapWrapper.set(this.protoLocals, templateName, null);
  }
  ElementBinder bindElement(ElementBinder parent, int distanceToParent,
      ProtoElementInjector protoElementInjector,
      [DirectiveMetadata componentDirective = null,
      DirectiveMetadata viewportDirective = null]) {
    var elBinder = new ElementBinder(this.elementBinders.length, parent,
        distanceToParent, protoElementInjector, componentDirective,
        viewportDirective);
    ListWrapper.push(this.elementBinders, elBinder);
    return elBinder;
  }
  bindTextNode(int indexInParent, AST expression) {
    var elBinder = this.elementBinders[this.elementBinders.length - 1];
    if (isBlank(elBinder.textNodeIndices)) {
      elBinder.textNodeIndices = ListWrapper.create();
    }
    ListWrapper.push(elBinder.textNodeIndices, indexInParent);
    var memento = this.textNodesWithBindingCount++;
    ListWrapper.push(
        this.bindingRecords, new BindingRecord(expression, memento, null));
  }
  bindElementProperty(AST expression, String setterName, SetterFn setter) {
    var elBinder = this.elementBinders[this.elementBinders.length - 1];
    if (!elBinder.hasElementPropertyBindings) {
      elBinder.hasElementPropertyBindings = true;
      this.elementsWithBindingCount++;
    }
    var memento = new ElementBindingMemento(
        this.elementsWithBindingCount - 1, setterName, setter);
    ListWrapper.push(
        this.bindingRecords, new BindingRecord(expression, memento, null));
  }
  bindEvent(String eventName, AST expression, [int directiveIndex = -1]) {
    var elBinder = this.elementBinders[this.elementBinders.length - 1];
    var events = elBinder.events;
    if (isBlank(events)) {
      events = StringMapWrapper.create();
      elBinder.events = events;
    }
    var event = StringMapWrapper.get(events, eventName);
    if (isBlank(event)) {
      event = MapWrapper.create();
      StringMapWrapper.set(events, eventName, event);
    }
    MapWrapper.set(event, directiveIndex, expression);
  }
  bindDirectiveProperty(
      num directiveIndex, AST expression, String setterName, SetterFn setter) {
    var bindingMemento = new DirectiveBindingMemento(
        this.elementBinders.length - 1, directiveIndex, setterName, setter);
    var directiveMemento = DirectiveMemento.get(bindingMemento);
    ListWrapper.push(this.bindingRecords,
        new BindingRecord(expression, bindingMemento, directiveMemento));
  }
  static ProtoView createRootProtoView(ProtoView protoView, insertionElement,
      DirectiveMetadata rootComponentAnnotatedType,
      ProtoChangeDetector protoChangeDetector,
      ShadowDomStrategy shadowDomStrategy) {
    DOM.addClass(insertionElement, NG_BINDING_CLASS);
    var cmpType = rootComponentAnnotatedType.type;
    var rootProtoView =
        new ProtoView(insertionElement, protoChangeDetector, shadowDomStrategy);
    rootProtoView.instantiateInPlace = true;
    var binder = rootProtoView.bindElement(
        null, 0, new ProtoElementInjector(null, 0, [cmpType], true));
    binder.componentDirective = rootComponentAnnotatedType;
    binder.nestedProtoView = protoView;
    shadowDomStrategy
        .shimAppElement(rootComponentAnnotatedType, insertionElement);
    return rootProtoView;
  }
}
class ElementBindingMemento {
  int _elementIndex;
  String _setterName;
  SetterFn _setter;
  ElementBindingMemento(int elementIndex, String setterName, SetterFn setter) {
    this._elementIndex = elementIndex;
    this._setterName = setterName;
    this._setter = setter;
  }
  invoke(ChangeRecord record, List bindElements) {
    var element = bindElements[this._elementIndex];
    this._setter(element, record.currentValue);
  }
}
class DirectiveBindingMemento {
  int _elementInjectorIndex;
  int _directiveIndex;
  String _setterName;
  SetterFn _setter;
  DirectiveBindingMemento(num elementInjectorIndex, num directiveIndex,
      String setterName, SetterFn setter) {
    this._elementInjectorIndex = elementInjectorIndex;
    this._directiveIndex = directiveIndex;
    this._setterName = setterName;
    this._setter = setter;
  }
  invoke(ChangeRecord record, List<ElementInjector> elementInjectors) {
    ElementInjector elementInjector = elementInjectors[
        this._elementInjectorIndex];
    var directive = elementInjector.getDirectiveAtIndex(this._directiveIndex);
    this._setter(directive, record.currentValue);
  }
}
var _directiveMementos = MapWrapper.create();
class DirectiveMemento {
  num _elementInjectorIndex;
  num _directiveIndex;
  DirectiveMemento(num elementInjectorIndex, num directiveIndex) {
    this._elementInjectorIndex = elementInjectorIndex;
    this._directiveIndex = directiveIndex;
  }
  static get(DirectiveBindingMemento memento) {
    var elementInjectorIndex = memento._elementInjectorIndex;
    var directiveIndex = memento._directiveIndex;
    var id = elementInjectorIndex * 100 + directiveIndex;
    if (!MapWrapper.contains(_directiveMementos, id)) {
      MapWrapper.set(_directiveMementos, id,
          new DirectiveMemento(elementInjectorIndex, directiveIndex));
    }
    return MapWrapper.get(_directiveMementos, id);
  }
  directive(List<ElementInjector> elementInjectors) {
    ElementInjector elementInjector = elementInjectors[
        this._elementInjectorIndex];
    return elementInjector.getDirectiveAtIndex(this._directiveIndex);
  }
  directiveBinding(List<ElementInjector> elementInjectors) {
    ElementInjector elementInjector = elementInjectors[
        this._elementInjectorIndex];
    return elementInjector.getDirectiveBindingAtIndex(this._directiveIndex);
  }
}
class PropertyUpdate {
  var currentValue;
  var previousValue;
  PropertyUpdate(currentValue, previousValue) {
    this.currentValue = currentValue;
    this.previousValue = previousValue;
  }
  static createWithoutPrevious(currentValue) {
    return new PropertyUpdate(currentValue, uninitialized);
  }
}
