library angular2.src.core.events.event_manager;

import 'package:angular2/src/facade/lang.dart'
    show isBlank, BaseException, isPresent, StringWrapper;
import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import 'package:angular2/src/facade/collection.dart'
    show List, ListWrapper, MapWrapper;
import 'package:angular2/src/core/zone/vm_turn_zone.dart' show VmTurnZone;

var BUBBLE_SYMBOL = '^';
class EventManager {
  List<EventManagerPlugin> _plugins;
  VmTurnZone _zone;
  EventManager(List<EventManagerPlugin> plugins, VmTurnZone zone) {
    this._zone = zone;
    this._plugins = plugins;
    for (var i = 0; i < plugins.length; i++) {
      plugins[i].manager = this;
    }
  }
  addEventListener(element, String eventName, Function handler) {
    var shouldSupportBubble = eventName[0] == BUBBLE_SYMBOL;
    if (shouldSupportBubble) {
      eventName = StringWrapper.substring(eventName, 1);
    }
    var plugin = this._findPluginFor(eventName);
    plugin.addEventListener(element, eventName, handler, shouldSupportBubble);
  }
  VmTurnZone getZone() {
    return this._zone;
  }
  EventManagerPlugin _findPluginFor(String eventName) {
    var plugins = this._plugins;
    for (var i = 0; i < plugins.length; i++) {
      var plugin = plugins[i];
      if (plugin.supports(eventName)) {
        return plugin;
      }
    }
    throw new BaseException(
        '''No event manager plugin found for event ${eventName}''');
  }
}
class EventManagerPlugin {
  EventManager manager;
  bool supports(String eventName) {
    return false;
  }
  addEventListener(
      element, String eventName, Function handler, bool shouldSupportBubble) {
    throw "not implemented";
  }
}
class DomEventsPlugin extends EventManagerPlugin {
  EventManager manager;
  bool supports(String eventName) {
    return true;
  }
  addEventListener(
      element, String eventName, Function handler, bool shouldSupportBubble) {
    var outsideHandler = shouldSupportBubble
        ? DomEventsPlugin.bubbleCallback(element, handler, this.manager._zone)
        : DomEventsPlugin.sameElementCallback(
            element, handler, this.manager._zone);
    this.manager._zone.runOutsideAngular(() {
      DOM.on(element, eventName, outsideHandler);
    });
  }
  static sameElementCallback(element, handler, zone) {
    return (event) {
      if (identical(event.target, element)) {
        zone.run(() {
          return handler(event);
        });
      }
    };
  }
  static bubbleCallback(element, handler, zone) {
    return (event) {
      return zone.run(() {
        return handler(event);
      });
    };
  }
}
