library angular2.src.core.life_cycle.life_cycle;

import 'package:angular2/di.dart' show Injectable;
import 'package:angular2/change_detection.dart' show ChangeDetector;
import 'package:angular2/src/core/zone/vm_turn_zone.dart' show VmTurnZone;
import 'package:angular2/src/core/exception_handler.dart' show ExceptionHandler;
import 'package:angular2/src/facade/lang.dart' show isPresent;

@Injectable()
class LifeCycle {
  var _errorHandler;
  ChangeDetector _changeDetector;
  bool _enforceNoNewChanges;
  LifeCycle(ExceptionHandler exceptionHandler,
      [ChangeDetector changeDetector = null,
      bool enforceNoNewChanges = false]) {
    this._errorHandler = (exception, stackTrace) {
      exceptionHandler.call(exception, stackTrace);
      throw exception;
    };
    this._changeDetector = changeDetector;
    this._enforceNoNewChanges = enforceNoNewChanges;
  }
  registerWith(VmTurnZone zone, [ChangeDetector changeDetector = null]) {
    if (isPresent(changeDetector)) {
      this._changeDetector = changeDetector;
    }
    zone.initCallbacks(onErrorHandler: this._errorHandler, onTurnDone: () {
      return this.tick();
    });
  }
  tick() {
    this._changeDetector.detectChanges();
    if (this._enforceNoNewChanges) {
      this._changeDetector.checkNoChanges();
    }
  }
}
