library angular2.src.directives.switch_dart;

import 'package:angular2/src/core/annotations/annotations.dart'
    show Decorator, Viewport;
import 'package:angular2/src/core/compiler/view_container.dart'
    show ViewContainer;
import 'package:angular2/src/core/dom/element.dart' show NgElement;
import 'package:angular2/src/facade/lang.dart'
    show isPresent, isBlank, normalizeBlank;
import 'package:angular2/src/facade/collection.dart'
    show ListWrapper, List, MapWrapper, Map;
import 'package:angular2/src/core/annotations/visibility.dart' show Parent;

@Decorator(selector: '[switch]', bind: const {'value': 'switch'})
class Switch {
  dynamic _switchValue;
  bool _useDefault;
  Map _valueViewContainers;
  List<ViewContainer> _activeViewContainers;
  Switch() {
    this._valueViewContainers = MapWrapper.create();
    this._activeViewContainers = ListWrapper.create();
    this._useDefault = false;
  }
  set value(value) {
    this._emptyAllActiveViewContainers();
    this._useDefault = false;
    var containers = MapWrapper.get(this._valueViewContainers, value);
    if (isBlank(containers)) {
      this._useDefault = true;
      containers = normalizeBlank(
          MapWrapper.get(this._valueViewContainers, _whenDefault));
    }
    this._activateViewContainers(containers);
    this._switchValue = value;
  }
  _onWhenValueChanged(oldWhen, newWhen, ViewContainer viewContainer) {
    this._deregisterViewContainer(oldWhen, viewContainer);
    this._registerViewContainer(newWhen, viewContainer);
    if (identical(oldWhen, this._switchValue)) {
      viewContainer.remove();
      ListWrapper.remove(this._activeViewContainers, viewContainer);
    } else if (identical(newWhen, this._switchValue)) {
      if (this._useDefault) {
        this._useDefault = false;
        this._emptyAllActiveViewContainers();
      }
      viewContainer.create();
      ListWrapper.push(this._activeViewContainers, viewContainer);
    }
    if (identical(this._activeViewContainers.length, 0) && !this._useDefault) {
      this._useDefault = true;
      this._activateViewContainers(
          MapWrapper.get(this._valueViewContainers, _whenDefault));
    }
  }
  _emptyAllActiveViewContainers() {
    var activeContainers = this._activeViewContainers;
    for (var i = 0; i < activeContainers.length; i++) {
      activeContainers[i].remove();
    }
    this._activeViewContainers = ListWrapper.create();
  }
  _activateViewContainers(List<ViewContainer> containers) {
    if (isPresent(containers)) {
      for (var i = 0; i < containers.length; i++) {
        containers[i].create();
      }
      this._activeViewContainers = containers;
    }
  }
  _registerViewContainer(value, ViewContainer container) {
    var containers = MapWrapper.get(this._valueViewContainers, value);
    if (isBlank(containers)) {
      containers = ListWrapper.create();
      MapWrapper.set(this._valueViewContainers, value, containers);
    }
    ListWrapper.push(containers, container);
  }
  _deregisterViewContainer(value, ViewContainer container) {
    if (value == _whenDefault) return;
    var containers = MapWrapper.get(this._valueViewContainers, value);
    if (containers.length == 1) {
      MapWrapper.delete(this._valueViewContainers, value);
    } else {
      ListWrapper.remove(containers, container);
    }
  }
}
@Viewport(selector: '[switch-when]', bind: const {'when': 'switch-when'})
class SwitchWhen {
  dynamic _value;
  Switch _switch;
  ViewContainer _viewContainer;
  SwitchWhen(
      NgElement el, ViewContainer viewContainer, @Parent() Switch sswitch) {
    this._value = _whenDefault;
    this._switch = sswitch;
    this._viewContainer = viewContainer;
  }
  set when(value) {
    this._switch._onWhenValueChanged(this._value, value, this._viewContainer);
    this._value = value;
  }
}
@Viewport(selector: '[switch-default]')
class SwitchDefault {
  SwitchDefault(ViewContainer viewContainer, @Parent() Switch sswitch) {
    sswitch._registerViewContainer(_whenDefault, viewContainer);
  }
}
var _whenDefault = new Object();
