library angular2.src.forms.directives;

import 'package:angular2/angular2.dart'
    show Template, Component, Decorator, Ancestor, onChange, PropertySetter;
import 'package:angular2/di.dart' show Optional;
import 'package:angular2/src/facade/lang.dart'
    show isBlank, isPresent, isString, CONST;
import 'package:angular2/src/facade/collection.dart'
    show StringMapWrapper, ListWrapper;
import './model.dart' show ControlGroup, Control;
import './validators.dart' show Validators;

@Decorator(
    selector: '[control]',
    events: const {
  'change': 'onChange(\$event.target.value)',
  'input': 'onChange(\$event.target.value)'
})
class DefaultValueAccessor {
  Function _setValueProperty;
  Function onChange;
  DefaultValueAccessor(@PropertySetter('value') Function setValueProperty)
      : super() {
    this._setValueProperty = setValueProperty;
    this.onChange = (_) {};
  }
  writeValue(value) {
    this._setValueProperty(value);
  }
}
@Decorator(
    selector: 'input[type=checkbox]',
    events: const {'change': 'onChange(\$event.target.checked)'})
class CheckboxControlValueAccessor {
  Function _setCheckedProperty;
  Function onChange;
  CheckboxControlValueAccessor(ControlDirective cd,
      @PropertySetter('checked') Function setCheckedProperty)
      : super() {
    this._setCheckedProperty = setCheckedProperty;
    this.onChange = (_) {};
    cd.valueAccessor = this;
  }
  writeValue(value) {
    this._setCheckedProperty(value);
  }
}
@Decorator(
    lifecycle: const [onChange],
    selector: '[control]',
    bind: const {'controlName': 'control'})
class ControlDirective {
  ControlGroupDirective _groupDirective;
  String controlName;
  dynamic valueAccessor;
  Function validator;
  ControlDirective(@Ancestor() ControlGroupDirective groupDirective,
      DefaultValueAccessor valueAccessor) {
    this._groupDirective = groupDirective;
    this.controlName = null;
    this.valueAccessor = valueAccessor;
    this.validator = Validators.nullValidator;
  }
  onChange(_) {
    this._initialize();
  }
  _initialize() {
    this._groupDirective.addDirective(this);
    var c = this._control();
    c.validator = Validators.compose([c.validator, this.validator]);
    this._updateDomValue();
    this._setUpUpdateControlValue();
  }
  _updateDomValue() {
    this.valueAccessor.writeValue(this._control().value);
  }
  _setUpUpdateControlValue() {
    this.valueAccessor.onChange = (newValue) {
      return this._control().updateValue(newValue);
    };
  }
  _control() {
    return this._groupDirective.findControl(this.controlName);
  }
}
@Decorator(
    selector: '[control-group]', bind: const {'controlGroup': 'control-group'})
class ControlGroupDirective {
  ControlGroupDirective _groupDirective;
  String _controlGroupName;
  ControlGroup _controlGroup;
  List<ControlDirective> _directives;
  ControlGroupDirective(
      @Optional() @Ancestor() ControlGroupDirective groupDirective)
      : super() {
    this._groupDirective = groupDirective;
    this._directives = ListWrapper.create();
  }
  set controlGroup(controlGroup) {
    if (isString(controlGroup)) {
      this._controlGroupName = controlGroup;
    } else {
      this._controlGroup = controlGroup;
    }
    this._updateDomValue();
  }
  _updateDomValue() {
    ListWrapper.forEach(this._directives, (cd) {
      return cd._updateDomValue();
    });
  }
  addDirective(ControlDirective c) {
    ListWrapper.push(this._directives, c);
  }
  dynamic findControl(String name) {
    return this._getControlGroup().controls[name];
  }
  ControlGroup _getControlGroup() {
    if (isPresent(this._controlGroupName)) {
      return this._groupDirective.findControl(this._controlGroupName);
    } else {
      return this._controlGroup;
    }
  }
}
var FormDirectives = [
  ControlGroupDirective,
  ControlDirective,
  CheckboxControlValueAccessor,
  DefaultValueAccessor
];
