library angular2.src.forms.form_builder;

import 'package:angular2/src/facade/collection.dart'
    show StringMapWrapper, ListWrapper;
import 'package:angular2/src/facade/lang.dart' show isPresent;
import './model.dart' as modelModule;

class FormBuilder {
  modelModule.ControlGroup group(controlsConfig, [extra = null]) {
    var controls = this._reduceControls(controlsConfig);
    var optionals =
        isPresent(extra) ? StringMapWrapper.get(extra, "optionals") : null;
    var validator =
        isPresent(extra) ? StringMapWrapper.get(extra, "validator") : null;
    if (isPresent(validator)) {
      return new modelModule.ControlGroup(controls, optionals, validator);
    } else {
      return new modelModule.ControlGroup(controls, optionals);
    }
  }
  modelModule.Control control(value, [Function validator = null]) {
    if (isPresent(validator)) {
      return new modelModule.Control(value, validator);
    } else {
      return new modelModule.Control(value);
    }
  }
  _reduceControls(controlsConfig) {
    var controls = {};
    StringMapWrapper.forEach(controlsConfig, (controlConfig, controlName) {
      controls[controlName] = this._createControl(controlConfig);
    });
    return controls;
  }
  _createControl(controlConfig) {
    if (controlConfig is modelModule.Control ||
        controlConfig is modelModule.ControlGroup) {
      return controlConfig;
    } else if (ListWrapper.isList(controlConfig)) {
      var value = ListWrapper.get(controlConfig, 0);
      var validator = controlConfig.length > 1 ? controlConfig[1] : null;
      return this.control(value, validator);
    } else {
      return this.control(controlConfig);
    }
  }
}
