library angular2.src.forms.model;

import 'package:angular2/src/facade/lang.dart' show isPresent;
import 'package:angular2/src/facade/collection.dart' show Map, StringMapWrapper;
import './validators.dart' show Validators;

const VALID = "VALID";
const INVALID = "INVALID";
class AbstractControl {
  dynamic _value;
  String _status;
  var _errors;
  bool _updateNeeded;
  bool _pristine;
  ControlGroup _parent;
  Function validator;
  AbstractControl(Function validator) {
    this.validator = validator;
    this._updateNeeded = true;
    this._pristine = true;
  }
  get value {
    this._updateIfNeeded();
    return this._value;
  }
  get status {
    this._updateIfNeeded();
    return this._status;
  }
  get valid {
    this._updateIfNeeded();
    return identical(this._status, VALID);
  }
  get errors {
    this._updateIfNeeded();
    return this._errors;
  }
  get pristine {
    this._updateIfNeeded();
    return this._pristine;
  }
  get dirty {
    return !this.pristine;
  }
  setParent(parent) {
    this._parent = parent;
  }
  _updateIfNeeded() {}
  _updateParent() {
    if (isPresent(this._parent)) {
      this._parent._controlChanged();
    }
  }
}
class Control extends AbstractControl {
  Control(dynamic value, [Function validator = Validators.nullValidator])
      : super(validator) {
    this._value = value;
  }
  updateValue(dynamic value) {
    this._value = value;
    this._updateNeeded = true;
    this._pristine = false;
    this._updateParent();
  }
  _updateIfNeeded() {
    if (this._updateNeeded) {
      this._updateNeeded = false;
      this._errors = this.validator(this);
      this._status = isPresent(this._errors) ? INVALID : VALID;
    }
  }
}
class ControlGroup extends AbstractControl {
  var controls;
  var optionals;
  ControlGroup(controls,
      [optionals = null, Function validator = Validators.group])
      : super(validator) {
    this.controls = controls;
    this.optionals = isPresent(optionals) ? optionals : {};
    this._setParentForControls();
  }
  include(String controlName) {
    this._updateNeeded = true;
    StringMapWrapper.set(this.optionals, controlName, true);
  }
  exclude(String controlName) {
    this._updateNeeded = true;
    StringMapWrapper.set(this.optionals, controlName, false);
  }
  contains(String controlName) {
    var c = StringMapWrapper.contains(this.controls, controlName);
    return c && this._included(controlName);
  }
  _setParentForControls() {
    StringMapWrapper.forEach(this.controls, (control, name) {
      control.setParent(this);
    });
  }
  _updateIfNeeded() {
    if (this._updateNeeded) {
      this._updateNeeded = false;
      this._value = this._reduceValue();
      this._pristine = this._reducePristine();
      this._errors = this.validator(this);
      this._status = isPresent(this._errors) ? INVALID : VALID;
    }
  }
  _reduceValue() {
    return this._reduceChildren({}, (acc, control, name) {
      acc[name] = control.value;
      return acc;
    });
  }
  _reducePristine() {
    return this._reduceChildren(true, (acc, control, name) {
      return acc && control.pristine;
    });
  }
  _reduceChildren(initValue, Function fn) {
    var res = initValue;
    StringMapWrapper.forEach(this.controls, (control, name) {
      if (this._included(name)) {
        res = fn(res, control, name);
      }
    });
    return res;
  }
  _controlChanged() {
    this._updateNeeded = true;
    this._updateParent();
  }
  bool _included(String controlName) {
    var isOptional = StringMapWrapper.contains(this.optionals, controlName);
    return !isOptional || StringMapWrapper.get(this.optionals, controlName);
  }
}
