library angular2.src.test_lib.test_injector;

import 'package:angular2/di.dart' show bind;
import 'package:angular2/src/core/compiler/compiler.dart'
    show Compiler, CompilerCache;
import 'package:angular2/src/reflection/reflection.dart'
    show Reflector, reflector;
import 'package:angular2/change_detection.dart'
    show Parser, Lexer, ChangeDetection, dynamicChangeDetection;
import 'package:angular2/src/core/exception_handler.dart' show ExceptionHandler;
import 'package:angular2/src/core/compiler/template_loader.dart'
    show TemplateLoader;
import 'package:angular2/src/core/compiler/template_resolver.dart'
    show TemplateResolver;
import 'package:angular2/src/core/compiler/directive_metadata_reader.dart'
    show DirectiveMetadataReader;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show ShadowDomStrategy, NativeShadowDomStrategy;
import 'package:angular2/src/core/compiler/xhr/xhr.dart' show XHR;
import 'package:angular2/src/mock/xhr_mock.dart' show XHRMock;
import 'package:angular2/src/core/compiler/component_url_mapper.dart'
    show ComponentUrlMapper;
import 'package:angular2/src/core/compiler/url_resolver.dart' show UrlResolver;
import 'package:angular2/src/core/compiler/style_url_resolver.dart'
    show StyleUrlResolver;
import 'package:angular2/src/core/compiler/style_inliner.dart'
    show StyleInliner;
import 'package:angular2/src/core/compiler/css_processor.dart'
    show CssProcessor;
import 'package:angular2/di.dart' show Injector;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper;
import 'package:angular2/src/facade/lang.dart' show FunctionWrapper;

_getRootBindings() {
  return [bind(Reflector).toValue(reflector)];
}
_getAppBindings() {
  return [
    bind(ShadowDomStrategy).toClass(NativeShadowDomStrategy),
    Compiler,
    CompilerCache,
    TemplateResolver,
    bind(ChangeDetection).toValue(dynamicChangeDetection),
    TemplateLoader,
    DirectiveMetadataReader,
    Parser,
    Lexer,
    ExceptionHandler,
    bind(XHR).toClass(XHRMock),
    ComponentUrlMapper,
    UrlResolver,
    StyleUrlResolver,
    StyleInliner,
    bind(CssProcessor).toFactory(() {
      return new CssProcessor(null);
    }, [])
  ];
}
createTestInjector(List bindings) {
  var rootInjector = new Injector(_getRootBindings());
  return rootInjector
      .createChild(ListWrapper.concat(_getAppBindings(), bindings));
}
inject(List tokens, Function fn) {
  return new FunctionWithParamTokens(tokens, fn);
}
class FunctionWithParamTokens {
  List _tokens;
  Function _fn;
  FunctionWithParamTokens(List tokens, Function fn) {
    this._tokens = tokens;
    this._fn = fn;
  }
  execute(Injector injector) {
    var params = ListWrapper.map(this._tokens, (t) {
      return injector.get(t);
    });
    FunctionWrapper.apply(this._fn, params);
  }
}
