library angular2.test.core.compiler.css_processor_spec;

import 'package:angular2/test_lib.dart'
    show describe, it, expect, beforeEach, ddescribe, iit, xit, el;
import 'package:angular2/src/core/compiler/css_processor.dart'
    show CssProcessor, CssTransformer;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show ShadowDomStrategy;
import 'package:angular2/src/core/compiler/pipeline/compile_pipeline.dart'
    show CompilePipeline;
import 'package:angular2/src/core/compiler/pipeline/compile_element.dart'
    show CompileElement;
import 'package:angular2/src/core/compiler/pipeline/compile_step.dart'
    show CompileStep;
import 'package:angular2/src/core/compiler/pipeline/compile_control.dart'
    show CompileControl;
import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import 'package:angular2/src/core/annotations/annotations.dart' show Component;
import 'package:angular2/src/core/compiler/directive_metadata.dart'
    show DirectiveMetadata;
import 'package:angular2/src/facade/collection.dart' show ListWrapper;

main() {
  describe('CssProcessor', () {
    describe('compile step', () {
      createPipeline(CssProcessor cssProcessor, ShadowDomStrategy strategy,
          String templateUrl) {
        var annotation = new Component();
        var meta = new DirectiveMetadata(SomeComponent, annotation);
        return new CompilePipeline(
            [cssProcessor.getCompileStep(meta, strategy, templateUrl)]);
      }
      it('it should set ignoreBindings to true for style elements', () {
        var strategy = new FakeShadowDomStrategy(null);
        var cssProcessor = new CssProcessor(null);
        var pipeline = createPipeline(cssProcessor, strategy, 'http://base');
        var results = pipeline.process(el('<div><style></style></div>'));
        expect(results[0].ignoreBindings).toBe(false);
        expect(results[1].ignoreBindings).toBe(true);
      });
      it('should execute the strategy step for style elements', () {
        var processedEls = [];
        var compileStep = new MockStep((parent, current, control) {
          ListWrapper.push(processedEls, current.element);
        });
        var strategy = new FakeShadowDomStrategy(compileStep);
        var cssProcessor = new CssProcessor(null);
        var pipeline = createPipeline(cssProcessor, strategy, 'http://base');
        var results = pipeline.process(el('<div><style></style></div>'));
        expect(processedEls.length).toEqual(1);
        expect(processedEls[0]).toBe(results[1].element);
      });
      it('should apply the given transformers', () {
        var strategy = new FakeShadowDomStrategy(null);
        var cssProcessor = new CssProcessor([
          new FakeCssTransformer('/*transformer1 */'),
          new FakeCssTransformer('/*transformer2 */')
        ]);
        var pipeline = createPipeline(cssProcessor, strategy, 'http://base');
        var results = pipeline.process(el('<div><style></style></div>'));
        expect(results[1].element)
            .toHaveText('/*transformer1 *//*transformer2 */');
      });
    });
  });
}
class FakeShadowDomStrategy extends ShadowDomStrategy {
  CompileStep _compileStep;
  FakeShadowDomStrategy(CompileStep compileStep) : super() {
    this._compileStep = compileStep;
  }
  CompileStep getStyleCompileStep(
      DirectiveMetadata cmpMetadata, String templateUrl) {
    return this._compileStep;
  }
}
class MockStep extends CompileStep {
  Function processClosure;
  MockStep(process) : super() {
    this.processClosure = process;
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    this.processClosure(parent, current, control);
  }
}
class FakeCssTransformer extends CssTransformer {
  String _css;
  FakeCssTransformer(String css) : super() {
    this._css = css;
  }
  transform(styleEl) {
    var cssText = DOM.getText(styleEl);
    cssText += this._css;
    DOM.setText(styleEl, cssText);
  }
}
class SomeComponent {}
