library angular2.test.core.compiler.pipeline.element_binding_marker_spec;

import 'package:angular2/test_lib.dart'
    show describe, beforeEach, it, expect, iit, ddescribe, el;
import 'package:angular2/src/facade/lang.dart' show isPresent;
import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import 'package:angular2/src/facade/collection.dart' show MapWrapper;
import 'package:angular2/src/core/compiler/pipeline/element_binding_marker.dart'
    show ElementBindingMarker;
import 'package:angular2/src/core/compiler/pipeline/compile_pipeline.dart'
    show CompilePipeline;
import 'package:angular2/src/core/compiler/pipeline/compile_element.dart'
    show CompileElement;
import 'package:angular2/src/core/compiler/pipeline/compile_step.dart'
    show CompileStep;
import 'package:angular2/src/core/compiler/pipeline/compile_control.dart'
    show CompileControl;
import 'package:angular2/src/core/compiler/directive_metadata_reader.dart'
    show DirectiveMetadataReader;
import 'package:angular2/src/core/annotations/annotations.dart'
    show Viewport, Decorator, Component;

main() {
  describe('ElementBindingMarker', () {
    createPipeline({textNodeBindings, propertyBindings, variableBindings,
        eventBindings, directives, ignoreBindings}) {
      var reader = new DirectiveMetadataReader();
      return new CompilePipeline([
        new MockStep((parent, current, control) {
          if (isPresent(textNodeBindings)) {
            current.textNodeBindings = textNodeBindings;
          }
          if (isPresent(propertyBindings)) {
            current.propertyBindings = propertyBindings;
          }
          if (isPresent(variableBindings)) {
            current.variableBindings = variableBindings;
          }
          if (isPresent(eventBindings)) {
            current.eventBindings = eventBindings;
          }
          if (isPresent(ignoreBindings)) {
            current.ignoreBindings = ignoreBindings;
          }
          if (isPresent(directives)) {
            for (var i = 0; i < directives.length; i++) {
              current.addDirective(reader.read(directives[i]));
            }
          }
        }),
        new ElementBindingMarker()
      ]);
    }
    it('should not mark empty elements', () {
      var results = createPipeline().process(el('<div></div>'));
      assertBinding(results[0], false);
    });
    it('should not mark elements when ignoreBindings is true', () {
      var textNodeBindings = MapWrapper.create();
      MapWrapper.set(textNodeBindings, 0, 'expr');
      var results = createPipeline(
              textNodeBindings: textNodeBindings, ignoreBindings: true)
          .process(el('<div></div>'));
      assertBinding(results[0], false);
    });
    it('should mark elements with text node bindings', () {
      var textNodeBindings = MapWrapper.create();
      MapWrapper.set(textNodeBindings, 0, 'expr');
      var results = createPipeline(textNodeBindings: textNodeBindings)
          .process(el('<div></div>'));
      assertBinding(results[0], true);
    });
    it('should mark elements with property bindings', () {
      var propertyBindings = MapWrapper.createFromStringMap({'a': 'expr'});
      var results = createPipeline(propertyBindings: propertyBindings)
          .process(el('<div></div>'));
      assertBinding(results[0], true);
    });
    it('should mark elements with variable bindings', () {
      var variableBindings = MapWrapper.createFromStringMap({'a': 'expr'});
      var results = createPipeline(variableBindings: variableBindings)
          .process(el('<div></div>'));
      assertBinding(results[0], true);
    });
    it('should mark elements with event bindings', () {
      var eventBindings = MapWrapper.createFromStringMap({'click': 'expr'});
      var results = createPipeline(eventBindings: eventBindings)
          .process(el('<div></div>'));
      assertBinding(results[0], true);
    });
    it('should mark elements with decorator directives', () {
      var results = createPipeline(directives: [SomeDecoratorDirective])
          .process(el('<div></div>'));
      assertBinding(results[0], true);
    });
    it('should mark elements with template directives', () {
      var results = createPipeline(directives: [SomeViewportDirective])
          .process(el('<div></div>'));
      assertBinding(results[0], true);
    });
    it('should mark elements with component directives', () {
      var results = createPipeline(directives: [SomeComponentDirective])
          .process(el('<div></div>'));
      assertBinding(results[0], true);
    });
  });
}
assertBinding(pipelineElement, shouldBePresent) {
  expect(pipelineElement.hasBindings).toBe(shouldBePresent);
  expect(DOM.hasClass(pipelineElement.element, 'ng-binding'))
      .toBe(shouldBePresent);
}
class MockStep extends CompileStep {
  Function processClosure;
  MockStep(process) : super() {
    this.processClosure = process;
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    this.processClosure(parent, current, control);
  }
}
@Viewport()
class SomeViewportDirective {}
@Component()
class SomeComponentDirective {}
@Decorator()
class SomeDecoratorDirective {}
