library angular2.test.core.compiler.shadow_dom.shadow_dom_emulation_integration_spec;

import 'package:angular2/test_lib.dart'
    show
        AsyncTestCompleter,
        beforeEach,
        ddescribe,
        describe,
        el,
        expect,
        iit,
        inject,
        it,
        xit;
import 'package:angular2/src/facade/collection.dart'
    show StringMapWrapper, List;
import 'package:angular2/src/facade/lang.dart' show Type;
import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import 'package:angular2/di.dart' show Injector;
import 'package:angular2/change_detection.dart'
    show Lexer, Parser, ChangeDetector, dynamicChangeDetection;
import 'package:angular2/src/core/exception_handler.dart' show ExceptionHandler;
import 'package:angular2/src/core/compiler/compiler.dart'
    show Compiler, CompilerCache;
import 'package:angular2/src/core/life_cycle/life_cycle.dart' show LifeCycle;
import 'package:angular2/src/core/compiler/directive_metadata_reader.dart'
    show DirectiveMetadataReader;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show
        ShadowDomStrategy,
        NativeShadowDomStrategy,
        EmulatedScopedShadowDomStrategy,
        EmulatedUnscopedShadowDomStrategy;
import 'package:angular2/src/core/compiler/template_loader.dart'
    show TemplateLoader;
import 'package:angular2/src/core/compiler/component_url_mapper.dart'
    show ComponentUrlMapper;
import 'package:angular2/src/core/compiler/url_resolver.dart' show UrlResolver;
import 'package:angular2/src/core/compiler/style_url_resolver.dart'
    show StyleUrlResolver;
import 'package:angular2/src/core/compiler/style_inliner.dart'
    show StyleInliner;
import 'package:angular2/src/core/compiler/css_processor.dart'
    show CssProcessor;
import 'package:angular2/src/mock/template_resolver_mock.dart'
    show MockTemplateResolver;
import 'package:angular2/src/core/annotations/annotations.dart'
    show Decorator, Component, Viewport;
import 'package:angular2/src/core/annotations/template.dart' show Template;
import 'package:angular2/src/core/compiler/view_container.dart'
    show ViewContainer;
import 'package:angular2/src/dom/browser_adapter.dart' show BrowserDomAdapter;

main() {
  BrowserDomAdapter.makeCurrent();
  describe('integration tests', () {
    var urlResolver = new UrlResolver();
    var styleUrlResolver = new StyleUrlResolver(urlResolver);
    var styleInliner = new StyleInliner(null, styleUrlResolver, urlResolver);
    var strategies = {
      "scoped": new EmulatedScopedShadowDomStrategy(
          styleInliner, styleUrlResolver, DOM.createElement('div')),
      "unscoped": new EmulatedUnscopedShadowDomStrategy(
          styleUrlResolver, DOM.createElement('div'))
    };
    if (DOM.supportsNativeShadowDOM()) {
      StringMapWrapper.set(
          strategies, "native", new NativeShadowDomStrategy(styleUrlResolver));
    }
    StringMapWrapper.forEach(strategies, (strategy, name) {
      describe('''${name} shadow dom strategy''', () {
        var compiler;
        var tplResolver;
        beforeEach(() {
          tplResolver = new MockTemplateResolver();
          compiler = new Compiler(dynamicChangeDetection,
              new TemplateLoader(null, null), new DirectiveMetadataReader(),
              new Parser(new Lexer()), new CompilerCache(), strategy,
              tplResolver, new ComponentUrlMapper(), urlResolver,
              new CssProcessor(null));
        });
        compile(template, List<Type> directives, assertions) {
          tplResolver.setTemplate(
              MyComp, new Template(inline: template, directives: directives));
          compiler.compile(MyComp).then(createView).then((view) {
            var lc = new LifeCycle(
                new ExceptionHandler(), view.changeDetector, false);
            assertions(view, lc);
          });
        }
        it('should support simple components', inject([
          AsyncTestCompleter
        ], (async) {
          var temp = '<simple>' + '<div>A</div>' + '</simple>';
          compile(temp, [Simple], (view, lc) {
            expect(view.nodes).toHaveText('SIMPLE(A)');
            async.done();
          });
        }));
        it('should support multiple content tags', inject([
          AsyncTestCompleter
        ], (async) {
          var temp = '<multiple-content-tags>' +
              '<div>B</div>' +
              '<div>C</div>' +
              '<div class="left">A</div>' +
              '</multiple-content-tags>';
          compile(temp, [MultipleContentTagsComponent], (view, lc) {
            expect(view.nodes).toHaveText('(A, BC)');
            async.done();
          });
        }));
        it('should redistribute only direct children', inject([
          AsyncTestCompleter
        ], (async) {
          var temp = '<multiple-content-tags>' +
              '<div>B<div class="left">A</div></div>' +
              '<div>C</div>' +
              '</multiple-content-tags>';
          compile(temp, [MultipleContentTagsComponent], (view, lc) {
            expect(view.nodes).toHaveText('(, BAC)');
            async.done();
          });
        }));
        it("should redistribute direct child viewcontainers when the light dom changes",
            inject([AsyncTestCompleter], (async) {
          var temp = '<multiple-content-tags>' +
              '<div><div template="manual" class="left">A</div></div>' +
              '<div>B</div>' +
              '</multiple-content-tags>';
          compile(temp, [
            MultipleContentTagsComponent,
            ManualViewportDirective
          ], (view, lc) {
            var dir = view.elementInjectors[1].get(ManualViewportDirective);
            expect(view.nodes).toHaveText('(, B)');
            dir.show();
            lc.tick();
            expect(view.nodes).toHaveText('(, AB)');
            dir.hide();
            lc.tick();
            expect(view.nodes).toHaveText('(, B)');
            async.done();
          });
        }));
        it("should redistribute when the light dom changes", inject([
          AsyncTestCompleter
        ], (async) {
          var temp = '<multiple-content-tags>' +
              '<div template="manual" class="left">A</div>' +
              '<div>B</div>' +
              '</multiple-content-tags>';
          compile(temp, [
            MultipleContentTagsComponent,
            ManualViewportDirective
          ], (view, lc) {
            var dir = view.elementInjectors[1].get(ManualViewportDirective);
            expect(view.nodes).toHaveText('(, B)');
            dir.show();
            lc.tick();
            expect(view.nodes).toHaveText('(A, B)');
            dir.hide();
            lc.tick();
            expect(view.nodes).toHaveText('(, B)');
            async.done();
          });
        }));
        it("should support nested components", inject([
          AsyncTestCompleter
        ], (async) {
          var temp = '<outer-with-indirect-nested>' +
              '<div>A</div>' +
              '<div>B</div>' +
              '</outer-with-indirect-nested>';
          compile(temp, [OuterWithIndirectNestedComponent], (view, lc) {
            expect(view.nodes).toHaveText('OUTER(SIMPLE(AB))');
            async.done();
          });
        }));
        it("should support nesting with content being direct child of a nested component",
            inject([AsyncTestCompleter], (async) {
          var temp = '<outer>' +
              '<div template="manual" class="left">A</div>' +
              '<div>B</div>' +
              '<div>C</div>' +
              '</outer>';
          compile(temp, [OuterComponent, ManualViewportDirective], (view, lc) {
            var dir = view.elementInjectors[1].get(ManualViewportDirective);
            expect(view.nodes).toHaveText('OUTER(INNER(INNERINNER(,BC)))');
            dir.show();
            lc.tick();
            expect(view.nodes).toHaveText('OUTER(INNER(INNERINNER(A,BC)))');
            async.done();
          });
        }));
        it('should redistribute when the shadow dom changes', inject([
          AsyncTestCompleter
        ], (async) {
          var temp = '<conditional-content>' +
              '<div class="left">A</div>' +
              '<div>B</div>' +
              '<div>C</div>' +
              '</conditional-content>';
          compile(temp, [
            ConditionalContentComponent,
            AutoViewportDirective
          ], (view, lc) {
            var cmp = view.elementInjectors[0].get(ConditionalContentComponent);
            expect(view.nodes).toHaveText('(, ABC)');
            cmp.showLeft();
            lc.tick();
            expect(view.nodes).toHaveText('(A, BC)');
            cmp.hideLeft();
            lc.tick();
            expect(view.nodes).toHaveText('(, ABC)');
            async.done();
          });
        }));
      });
    });
  });
}
class TestDirectiveMetadataReader extends DirectiveMetadataReader {
  var shadowDomStrategy;
  TestDirectiveMetadataReader(shadowDomStrategy) : super() {
    this.shadowDomStrategy = shadowDomStrategy;
  }
  ShadowDomStrategy parseShadowDomStrategy(Component annotation) {
    return this.shadowDomStrategy;
  }
}
@Viewport(selector: '[manual]')
class ManualViewportDirective {
  var viewContainer;
  ManualViewportDirective(ViewContainer viewContainer) {
    this.viewContainer = viewContainer;
  }
  show() {
    this.viewContainer.create();
  }
  hide() {
    this.viewContainer.remove(0);
  }
}
@Viewport(selector: '[auto]', bind: const {'auto': 'auto'})
class AutoViewportDirective {
  var viewContainer;
  AutoViewportDirective(ViewContainer viewContainer) {
    this.viewContainer = viewContainer;
  }
  set auto(bool newValue) {
    if (newValue) {
      this.viewContainer.create();
    } else {
      this.viewContainer.remove(0);
    }
  }
}
@Component(selector: 'simple')
@Template(inline: 'SIMPLE(<content></content>)')
class Simple {}
@Component(selector: 'multiple-content-tags')
@Template(inline: '(<content select=".left"></content>, <content></content>)')
class MultipleContentTagsComponent {}
@Component(selector: 'conditional-content')
@Template(
    inline: '<div>(<div *auto="cond"><content select=".left"></content></div>, <content></content>)</div>',
    directives: const [AutoViewportDirective])
class ConditionalContentComponent {
  bool cond;
  ConditionalContentComponent() {
    this.cond = false;
  }
  showLeft() {
    this.cond = true;
  }
  hideLeft() {
    this.cond = false;
  }
}
@Component(selector: 'outer-with-indirect-nested')
@Template(
    inline: 'OUTER(<simple><div><content></content></div></simple>)',
    directives: const [Simple])
class OuterWithIndirectNestedComponent {}
@Component(selector: 'outer')
@Template(
    inline: 'OUTER(<inner><content></content></inner>)',
    directives: const [InnerComponent])
class OuterComponent {}
@Component(selector: 'inner')
@Template(
    inline: 'INNER(<innerinner><content></content></innerinner>)',
    directives: const [InnerInnerComponent])
class InnerComponent {}
@Component(selector: 'innerinner')
@Template(
    inline: 'INNERINNER(<content select=".left"></content>,<content></content>)')
class InnerInnerComponent {}
@Component(selector: 'my-comp')
@Template(
    directives: const [
  MultipleContentTagsComponent,
  ManualViewportDirective,
  ConditionalContentComponent,
  OuterWithIndirectNestedComponent,
  OuterComponent
])
class MyComp {}
createView(pv) {
  var view = pv.instantiate(null, null);
  view.hydrate(new Injector([]), null, null, {}, null);
  return view;
}
