library angular2.test.core.compiler.template_loader_spec;

import 'package:angular2/test_lib.dart'
    show
        AsyncTestCompleter,
        beforeEach,
        ddescribe,
        describe,
        el,
        expect,
        iit,
        inject,
        it,
        xit;
import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import 'package:angular2/src/core/compiler/template_loader.dart'
    show TemplateLoader;
import 'package:angular2/src/core/compiler/url_resolver.dart' show UrlResolver;
import 'package:angular2/src/core/annotations/template.dart' show Template;
import 'package:angular2/src/facade/async.dart' show PromiseWrapper;
import 'package:angular2/src/mock/xhr_mock.dart' show XHRMock;

main() {
  describe('TemplateLoader', () {
    var loader;
    var xhr;
    beforeEach(() {
      xhr = new XHRMock();
      loader = new TemplateLoader(xhr, new FakeUrlResolver());
    });
    it('should load inline templates synchronously', () {
      var template = new Template(inline: 'inline template');
      expect(DOM.content(loader.load(template))).toHaveText('inline template');
    });
    it('should load templates through XHR', inject([
      AsyncTestCompleter
    ], (async) {
      xhr.expect('base/foo', 'xhr template');
      var template = new Template(url: '/foo');
      loader.setBaseUrl(template, 'base');
      loader.load(template).then((el) {
        expect(DOM.content(el)).toHaveText('xhr template');
        async.done();
      });
      xhr.flush();
    }));
    it('should cache template loaded through XHR', inject([
      AsyncTestCompleter
    ], (async) {
      var firstEl;
      xhr.expect('base/foo', 'xhr template');
      var template = new Template(url: '/foo');
      loader.setBaseUrl(template, 'base');
      loader.load(template).then((el) {
        firstEl = el;
        return loader.load(template);
      }).then((el) {
        expect(el).toBe(firstEl);
        expect(DOM.content(el)).toHaveText('xhr template');
        async.done();
      });
      xhr.flush();
    }));
    it('should throw when no template is defined', () {
      var template = new Template(inline: null, url: null);
      expect(() {
        return loader.load(template);
      }).toThrowError(
          'Templates should have either their url or inline property set');
    });
    it('should return a rejected Promise when xhr loading fails', inject([
      AsyncTestCompleter
    ], (async) {
      xhr.expect('base/foo', null);
      var template = new Template(url: '/foo');
      loader.setBaseUrl(template, 'base');
      PromiseWrapper.then(loader.load(template), (_) {
        throw 'Unexpected response';
      }, (error) {
        expect(error).toEqual('Failed to load base/foo');
        async.done();
      });
      xhr.flush();
    }));
  });
}
class SomeComponent {}
class FakeUrlResolver extends UrlResolver {
  FakeUrlResolver() : super() {}
  String resolve(String baseUrl, String url) {
    return baseUrl + url;
  }
}
