library angular2.test.directives.non_bindable_spec;

import 'package:angular2/test_lib.dart'
    show
        AsyncTestCompleter,
        beforeEach,
        ddescribe,
        describe,
        el,
        expect,
        iit,
        inject,
        it,
        xit;
import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import 'package:angular2/di.dart' show Injector;
import 'package:angular2/change_detection.dart'
    show Lexer, Parser, ChangeDetector, dynamicChangeDetection;
import 'package:angular2/src/core/compiler/compiler.dart'
    show Compiler, CompilerCache;
import 'package:angular2/src/core/compiler/directive_metadata_reader.dart'
    show DirectiveMetadataReader;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show NativeShadowDomStrategy;
import 'package:angular2/src/core/compiler/component_url_mapper.dart'
    show ComponentUrlMapper;
import 'package:angular2/src/core/compiler/url_resolver.dart' show UrlResolver;
import 'package:angular2/src/core/compiler/style_url_resolver.dart'
    show StyleUrlResolver;
import 'package:angular2/src/core/compiler/css_processor.dart'
    show CssProcessor;
import 'package:angular2/src/core/annotations/annotations.dart'
    show Decorator, Component;
import 'package:angular2/src/core/annotations/template.dart' show Template;
import 'package:angular2/src/core/compiler/template_loader.dart'
    show TemplateLoader;
import 'package:angular2/src/core/dom/element.dart' show NgElement;
import 'package:angular2/src/directives/non_bindable.dart' show NonBindable;
import 'package:angular2/src/mock/template_resolver_mock.dart'
    show MockTemplateResolver;

main() {
  describe('non-bindable', () {
    var view;
    var cd;
    var compiler;
    var component;
    var tplResolver;
    beforeEach(() {
      var urlResolver = new UrlResolver();
      tplResolver = new MockTemplateResolver();
      compiler = new Compiler(dynamicChangeDetection,
          new TemplateLoader(null, null), new DirectiveMetadataReader(),
          new Parser(new Lexer()), new CompilerCache(),
          new NativeShadowDomStrategy(new StyleUrlResolver(urlResolver)),
          tplResolver, new ComponentUrlMapper(), urlResolver,
          new CssProcessor(null));
    });
    createView(pv) {
      component = new TestComponent();
      view = pv.instantiate(null, null);
      view.hydrate(new Injector([]), null, null, component, null);
      cd = view.changeDetector;
    }
    compileWithTemplate(html) {
      var template =
          new Template(inline: html, directives: [NonBindable, TestDecorator]);
      tplResolver.setTemplate(TestComponent, template);
      return compiler.compile(TestComponent);
    }
    it('should not interpolate children', inject([AsyncTestCompleter], (async) {
      var template = '<div>{{text}}<span non-bindable>{{text}}</span></div>';
      compileWithTemplate(template).then((pv) {
        createView(pv);
        cd.detectChanges();
        expect(DOM.getText(view.nodes[0])).toEqual('foo{{text}}');
        async.done();
      });
    }));
    it('should ignore directives on child nodes', inject([
      AsyncTestCompleter
    ], (async) {
      var template =
          '<div non-bindable><span id=child test-dec>{{text}}</span></div>';
      compileWithTemplate(template).then((pv) {
        createView(pv);
        cd.detectChanges();
        var span = DOM.querySelector(view.nodes[0], '#child');
        expect(DOM.hasClass(span, 'compiled')).toBeFalsy();
        async.done();
      });
    }));
    it('should trigger directives on the same node', inject([
      AsyncTestCompleter
    ], (async) {
      var template =
          '<div><span id=child non-bindable test-dec>{{text}}</span></div>';
      compileWithTemplate(template).then((pv) {
        createView(pv);
        cd.detectChanges();
        var span = DOM.querySelector(view.nodes[0], '#child');
        expect(DOM.hasClass(span, 'compiled')).toBeTruthy();
        async.done();
      });
    }));
  });
}
@Component(selector: 'test-cmp')
class TestComponent {
  String text;
  TestComponent() {
    this.text = 'foo';
  }
}
@Decorator(selector: '[test-dec]')
class TestDecorator {
  TestDecorator(NgElement el) {
    DOM.addClass(el.domElement, 'compiled');
  }
}
