library angular2.test.directives.switch_spec;

import 'package:angular2/test_lib.dart'
    show
        AsyncTestCompleter,
        beforeEach,
        ddescribe,
        describe,
        el,
        expect,
        iit,
        inject,
        it,
        xit;
import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import 'package:angular2/di.dart' show Injector;
import 'package:angular2/change_detection.dart'
    show Lexer, Parser, dynamicChangeDetection;
import 'package:angular2/src/core/compiler/compiler.dart'
    show Compiler, CompilerCache;
import 'package:angular2/src/core/compiler/directive_metadata_reader.dart'
    show DirectiveMetadataReader;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart'
    show NativeShadowDomStrategy;
import 'package:angular2/src/core/compiler/component_url_mapper.dart'
    show ComponentUrlMapper;
import 'package:angular2/src/core/compiler/url_resolver.dart' show UrlResolver;
import 'package:angular2/src/core/compiler/style_url_resolver.dart'
    show StyleUrlResolver;
import 'package:angular2/src/core/compiler/css_processor.dart'
    show CssProcessor;
import 'package:angular2/src/core/annotations/annotations.dart' show Component;
import 'package:angular2/src/core/annotations/template.dart' show Template;
import 'package:angular2/core.dart' show TemplateLoader;
import 'package:angular2/src/directives/switch.dart'
    show Switch, SwitchWhen, SwitchDefault;
import 'package:angular2/src/mock/template_resolver_mock.dart'
    show MockTemplateResolver;

main() {
  describe('switch', () {
    var view;
    var cd;
    var compiler;
    var component;
    var tplResolver;
    beforeEach(() {
      var urlResolver = new UrlResolver();
      tplResolver = new MockTemplateResolver();
      compiler = new Compiler(dynamicChangeDetection,
          new TemplateLoader(null, null), new DirectiveMetadataReader(),
          new Parser(new Lexer()), new CompilerCache(),
          new NativeShadowDomStrategy(new StyleUrlResolver(urlResolver)),
          tplResolver, new ComponentUrlMapper(), urlResolver,
          new CssProcessor(null));
    });
    createView(pv) {
      component = new TestComponent();
      view = pv.instantiate(null, null);
      view.hydrate(new Injector([]), null, null, component, null);
      cd = view.changeDetector;
    }
    compileWithTemplate(html) {
      var template = new Template(
          inline: html, directives: [Switch, SwitchWhen, SwitchDefault]);
      tplResolver.setTemplate(TestComponent, template);
      return compiler.compile(TestComponent);
    }
    describe('switch value changes', () {
      it('should switch amongst when values', inject([
        AsyncTestCompleter
      ], (async) {
        var template = '<div>' +
            '<ul [switch]="switchValue">' +
            '<template [switch-when]="\'a\'"><li>when a</li></template>' +
            '<template [switch-when]="\'b\'"><li>when b</li></template>' +
            '</ul></div>';
        compileWithTemplate(template).then((pv) {
          createView(pv);
          cd.detectChanges();
          expect(DOM.getText(view.nodes[0])).toEqual('');
          component.switchValue = 'a';
          cd.detectChanges();
          expect(DOM.getText(view.nodes[0])).toEqual('when a');
          component.switchValue = 'b';
          cd.detectChanges();
          expect(DOM.getText(view.nodes[0])).toEqual('when b');
          async.done();
        });
      }));
      it('should switch amongst when values with fallback to default', inject([
        AsyncTestCompleter
      ], (async) {
        var template = '<div>' +
            '<ul [switch]="switchValue">' +
            '<li template="switch-when \'a\'">when a</li>' +
            '<li template="switch-default">when default</li>' +
            '</ul></div>';
        compileWithTemplate(template).then((pv) {
          createView(pv);
          cd.detectChanges();
          expect(DOM.getText(view.nodes[0])).toEqual('when default');
          component.switchValue = 'a';
          cd.detectChanges();
          expect(DOM.getText(view.nodes[0])).toEqual('when a');
          component.switchValue = 'b';
          cd.detectChanges();
          expect(DOM.getText(view.nodes[0])).toEqual('when default');
          async.done();
        });
      }));
      it('should support multiple whens with the same value', inject([
        AsyncTestCompleter
      ], (async) {
        var template = '<div>' +
            '<ul [switch]="switchValue">' +
            '<template [switch-when]="\'a\'"><li>when a1;</li></template>' +
            '<template [switch-when]="\'b\'"><li>when b1;</li></template>' +
            '<template [switch-when]="\'a\'"><li>when a2;</li></template>' +
            '<template [switch-when]="\'b\'"><li>when b2;</li></template>' +
            '<template [switch-default]><li>when default1;</li></template>' +
            '<template [switch-default]><li>when default2;</li></template>' +
            '</ul></div>';
        compileWithTemplate(template).then((pv) {
          createView(pv);
          cd.detectChanges();
          expect(DOM.getText(view.nodes[0]))
              .toEqual('when default1;when default2;');
          component.switchValue = 'a';
          cd.detectChanges();
          expect(DOM.getText(view.nodes[0])).toEqual('when a1;when a2;');
          component.switchValue = 'b';
          cd.detectChanges();
          expect(DOM.getText(view.nodes[0])).toEqual('when b1;when b2;');
          async.done();
        });
      }));
    });
    describe('when values changes', () {
      it('should switch amongst when values', inject([
        AsyncTestCompleter
      ], (async) {
        var template = '<div>' +
            '<ul [switch]="switchValue">' +
            '<template [switch-when]="when1"><li>when 1;</li></template>' +
            '<template [switch-when]="when2"><li>when 2;</li></template>' +
            '<template [switch-default]><li>when default;</li></template>' +
            '</ul></div>';
        compileWithTemplate(template).then((pv) {
          createView(pv);
          component.when1 = 'a';
          component.when2 = 'b';
          component.switchValue = 'a';
          cd.detectChanges();
          expect(DOM.getText(view.nodes[0])).toEqual('when 1;');
          component.switchValue = 'b';
          cd.detectChanges();
          expect(DOM.getText(view.nodes[0])).toEqual('when 2;');
          component.switchValue = 'c';
          cd.detectChanges();
          expect(DOM.getText(view.nodes[0])).toEqual('when default;');
          component.when1 = 'c';
          cd.detectChanges();
          expect(DOM.getText(view.nodes[0])).toEqual('when 1;');
          component.when1 = 'd';
          cd.detectChanges();
          expect(DOM.getText(view.nodes[0])).toEqual('when default;');
          async.done();
        });
      }));
    });
  });
}
@Component(selector: 'test-cmp')
class TestComponent {
  dynamic switchValue;
  dynamic when1;
  dynamic when2;
  TestComponent() {
    this.switchValue = null;
    this.when1 = null;
    this.when2 = null;
  }
}
